@@ -58,16 +58,21 @@ void do_housekeeping(void)
 {
 	wcsession *sptr, *ss;
 	wcsession *sessions_to_kill = NULL;
+	time_t the_time;
 
 	/*
 	 * Lock the session list, moving any candidates for euthanasia into
 	 * a separate list.
 	 */
+	the_time = 0;
 	CtdlLogResult(pthread_mutex_lock(&SessionListMutex));
 	for (sptr = SessionList; sptr != NULL; sptr = sptr->next) {
-
+		if (the_time == 0)
+			the_time = time(NULL);
 		/* Kill idle sessions */
-		if ((time(NULL) - (sptr->lastreq)) > (time_t) WEBCIT_TIMEOUT) {
+		if ((sptr->inuse == 0) && 
+		    ((the_time - (sptr->lastreq)) > (time_t) WEBCIT_TIMEOUT))
+		{
 			syslog(LOG_DEBUG, "Timeout session %d", sptr->wc_session);
 			sptr->killthis = 1;
 		}
@@ -208,6 +213,7 @@ wcsession *CreateSession(int Lockable, int Static, wcsession **wclist, ParsedHtt
 	memset(TheSession, 0, sizeof(wcsession));
 	TheSession->Hdr = Hdr;
 	TheSession->serv_sock = (-1);
+	TheSession->lastreq = time(NULL);;
 
 	pthread_setspecific(MyConKey, (void *)TheSession);
 	
@@ -488,6 +494,7 @@ void context_loop(ParsedHttpHdrs *Hdr)
 	struct timeval tx_start;
 	struct timeval tx_finish;
 	int session_may_be_reused = 1;
+	time_t now;
 	
 	gettimeofday(&tx_start, NULL);		/* start a stopwatch for performance timing */
 
@@ -560,7 +567,7 @@ void context_loop(ParsedHttpHdrs *Hdr)
 	/*
 	 * If there were no qualifying sessions, then create a new one.
 	 */
-	if (TheSession == NULL) {
+	if ((TheSession == NULL) || (TheSession->killthis != 0)) {
 		TheSession = CreateSession(1, 0, &SessionList, Hdr, &SessionListMutex);
 	}
 
@@ -585,11 +592,12 @@ void context_loop(ParsedHttpHdrs *Hdr)
 	/*
 	 * Bind to the session and perform the transaction
 	 */
+	now = time(NULL);;
 	CtdlLogResult(pthread_mutex_lock(&TheSession->SessionMutex));
 	pthread_setspecific(MyConKey, (void *)TheSession);
 	
-	TheSession->inuse = 1;					/* mark the session as bound */
-	TheSession->lastreq = time(NULL);			/* log */
+	TheSession->inuse = 1;				/* mark the session as bound */
+	TheSession->lastreq = now;			/* log */
 	TheSession->Hdr = Hdr;
 
 	/*
@@ -625,9 +633,9 @@ void context_loop(ParsedHttpHdrs *Hdr)
 	 * table from getting bombarded with new sessions when, for example, a web
 	 * spider crawls the site without using cookies.
 	 */
-	if ((session_may_be_reused) && (!WC->logged_in)) {
-		WC->wc_session = 0;			/* flag as available for re-use */
-		TheSession->selected_language = 0;	/* clear any non-default language setting */
+	if ((session_may_be_reused) && (!TheSession->logged_in)) {
+		TheSession->wc_session = 0;		/* flag as available for re-use */
+		TheSession->selected_language = -1;	/* clear any non-default language setting */
 	}
 
 	TheSession->Hdr = NULL;
