@@ -13,6 +13,8 @@
 #include "webcit.h"
 #include "webserver.h"
 
+CtxType CTX_FILELIST = CTX_NONE;
+
 extern void output_static(const char* What);
 
 extern char* static_dirs[];
@@ -38,22 +40,22 @@ void FreeFiles(void *vFile)
 /* -------------------------------------------------------------------------------- */
 void tmplput_FILE_NAME(StrBuf *Target, WCTemplputParams *TP)
 {
-	FileListStruct *F = (FileListStruct*) CTX;
+	FileListStruct *F = (FileListStruct*) CTX(CTX_FILELIST);
 	StrBufAppendTemplate(Target, TP, F->Filename, 0);
 }
 void tmplput_FILE_SIZE(StrBuf *Target, WCTemplputParams *TP)
 {
-	FileListStruct *F = (FileListStruct*) CTX;
+	FileListStruct *F = (FileListStruct*) CTX(CTX_FILELIST);
 	StrBufAppendPrintf(Target, "%ld", F->FileSize);
 }
 void tmplput_FILEMIMETYPE(StrBuf *Target, WCTemplputParams *TP)
 {
-	FileListStruct *F = (FileListStruct*) CTX;
+	FileListStruct *F = (FileListStruct*) CTX(CTX_FILELIST);
 	StrBufAppendTemplate(Target, TP, F->MimeType, 0);
 }
 void tmplput_FILE_COMMENT(StrBuf *Target, WCTemplputParams *TP)
 {
-	FileListStruct *F = (FileListStruct*) CTX;
+	FileListStruct *F = (FileListStruct*) CTX(CTX_FILELIST);
 	StrBufAppendTemplate(Target, TP, F->Comment, 0);
 }
 
@@ -61,7 +63,7 @@ void tmplput_FILE_COMMENT(StrBuf *Target, WCTemplputParams *TP)
 
 int Conditional_FILE_ISPIC(StrBuf *Target, WCTemplputParams *TP)
 {
-	FileListStruct *F = (FileListStruct*) CTX;
+	FileListStruct *F = (FileListStruct*) CTX(CTX_FILELIST);
 	return F->IsPic;
 }
 
@@ -420,6 +422,7 @@ void
 InitModule_DOWNLOAD
 (void)
 {
+	RegisterCTX(CTX_FILELIST);
 
 	RegisterIterator("ROOM:FILES", 0, NULL, LoadFileList,
 			 NULL, DeleteHash, CTX_FILELIST, CTX_NONE, 
@@ -461,7 +464,7 @@ InitModule_DOWNLOAD
 	RegisterNamespace("FILE:MIMETYPE", 0, 2, tmplput_FILEMIMETYPE, NULL, CTX_FILELIST);
 	RegisterNamespace("FILE:COMMENT", 0, 2, tmplput_FILE_COMMENT, NULL, CTX_FILELIST);
 
-	RegisterConditional(HKEY("COND:FILE:ISPIC"), 0, Conditional_FILE_ISPIC, CTX_FILELIST);
+	RegisterConditional("COND:FILE:ISPIC", 0, Conditional_FILE_ISPIC, CTX_FILELIST);
 
 	WebcitAddUrlHandler(HKEY("image"), "", 0, output_image, ANONYMOUS);
 	WebcitAddUrlHandler(HKEY("display_mime_icon"), "", 0, display_mime_icon , ANONYMOUS);
