@@ -1,21 +1,15 @@
 /*
  * Handles HTTP upload of graphics files into the system.
  *
- * Copyright (c) 1996-2011 by the citadel.org team
+ * Copyright (c) 1996-2012 by the citadel.org team
  *
  * This program is open source software.  You can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 3 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of the GNU General Public License, version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #include "webcit.h"
@@ -47,7 +41,6 @@ void do_graphics_upload(char *filename)
 	StrBuf *Line;
 	const char *MimeType;
 	wcsession *WCC = WC;
-	char buf[SIZ];
 	int bytes_remaining;
 	int pos = 0;
 	int thisblock;
@@ -78,18 +71,18 @@ void do_graphics_upload(char *filename)
 	while (bytes_remaining) {
 		thisblock = ((bytes_remaining > 4096) ? 4096 : bytes_remaining);
 		serv_printf("WRIT %d", thisblock);
-	StrBuf_ServGetln(Line);
-	if (GetServerStatusMsg(Line, NULL, 1, 7) != 7) {
+		StrBuf_ServGetln(Line);
+		if (GetServerStatusMsg(Line, NULL, 1, 7) != 7) {
 			serv_puts("UCLS 0");
 			StrBuf_ServGetln(Line);
 			display_main_menu();
 			FreeStrBuf(&Line);
 			return;
 		}
-		thisblock = extract_int(&buf[4], 0);
+		thisblock = extract_int(ChrPtr(Line) +4, 0);
 		serv_write(&ChrPtr(WCC->upload)[pos], thisblock);
-		pos = pos + thisblock;
-		bytes_remaining = bytes_remaining - thisblock;
+		pos += thisblock;
+		bytes_remaining -= thisblock;
 	}
 
 	serv_puts("UCLS 1");
