@@ -296,10 +296,10 @@ void output_html(const char *supplied_charset, int treat_as_wiki, int msgnum, St
 			&& (strcasecmp(charset, "UTF-8"))
 			&& (strcasecmp(charset, ""))
 	   ) {
-		syslog(9, "Converting %s to UTF-8\n", charset);
+		syslog(LOG_DEBUG, "Converting %s to UTF-8\n", charset);
 		ctdl_iconv_open("UTF-8", charset, &ic);
 		if (ic == (iconv_t)(-1) ) {
-			syslog(5, "%s:%d iconv_open() failed: %s\n",
+			syslog(LOG_WARNING, "%s:%d iconv_open() failed: %s\n",
 					__FILE__, __LINE__, strerror(errno));
 		}
 	}
@@ -414,9 +414,9 @@ void output_html(const char *supplied_charset, int treat_as_wiki, int msgnum, St
 			 * and then ended the message)
 			 */
 			if (!tag_end) {
-				syslog(9, "tag_end is null and ptr is:\n");
-				syslog(9, "%s\n", ptr);
-				syslog(9, "Theoretical bytes remaining: %d\n", (int)(msgend - ptr));
+				syslog(LOG_DEBUG, "tag_end is null and ptr is:\n");
+				syslog(LOG_DEBUG, "%s\n", ptr);
+				syslog(LOG_DEBUG, "Theoretical bytes remaining: %d\n", (int)(msgend - ptr));
 			}
 
 			src=strstr(ptr, "src=\"cid:");
@@ -641,7 +641,7 @@ void url(char *buf, size_t bufsize)
 	start = NULL;
 	len = strlen(buf);
 	if (len > bufsize) {
-		syslog(1, "URL: content longer than buffer!");
+		syslog(LOG_WARNING, "URL: content longer than buffer!");
 		return;
 	}
 	end = buf + len;
@@ -677,7 +677,7 @@ void url(char *buf, size_t bufsize)
 	
 	UrlLen = end - start;
 	if (UrlLen > sizeof(urlbuf)){
-		syslog(1, "URL: content longer than buffer!");
+		syslog(LOG_WARNING, "URL: content longer than buffer!");
 		return;
 	}
 	memcpy(urlbuf, start, UrlLen);
@@ -690,7 +690,7 @@ void url(char *buf, size_t bufsize)
 			  "%ca href=%c%s%c TARGET=%c%s%c%c%s%c/A%c",
 			  LB, QU, urlbuf, QU, QU, TARGET, QU, RB, urlbuf, LB, RB);
 	if (outpos >= sizeof(outbuf) - Offset) {
-		syslog(1, "URL: content longer than buffer!");
+		syslog(LOG_WARNING, "URL: content longer than buffer!");
 		return;
 	}
 
@@ -698,7 +698,7 @@ void url(char *buf, size_t bufsize)
 	if (TrailerLen > 0)
 		memcpy(outbuf + Offset + outpos, end, TrailerLen);
 	if (Offset + outpos + TrailerLen > bufsize) {
-		syslog(1, "URL: content longer than buffer!");
+		syslog(LOG_WARNING, "URL: content longer than buffer!");
 		return;
 	}
 	memcpy (buf, outbuf, Offset + outpos + TrailerLen);
