@@ -2,7 +2,8 @@
 #include "webserver.h"
 #include "dav.h"
 
-
+CtxType CTX_MAILSUM = CTX_NONE;
+CtxType CTX_MIME_ATACH = CTX_NONE;
 
 inline void CheckConvertBufs(struct wcsession *WCC)
 {
@@ -50,6 +51,7 @@ void DestroyMessageSummary(void *vMsg)
 	FreeStrBuf(&Msg->AllRcpt);
 	FreeStrBuf(&Msg->Room);
 	FreeStrBuf(&Msg->Rfca);
+	FreeStrBuf(&Msg->EnvTo);
 	FreeStrBuf(&Msg->OtherNode);
 
 	DeleteHash(&Msg->Attachments);	/* list of Attachments */
@@ -248,7 +250,7 @@ void examine_nhdr(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset)
 }
 int Conditional_ANONYMOUS_MESSAGE(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	return Msg->nhdr != 0;
 }
 
@@ -274,7 +276,7 @@ void examine_from(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset)
 }
 void tmplput_MAIL_SUMM_FROM(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	StrBufAppendTemplate(Target, TP, Msg->from, 0);
 }
 
@@ -294,7 +296,7 @@ void examine_subj(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset)
 }
 void tmplput_MAIL_SUMM_SUBJECT(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 
 	if (TP->Tokens->nParameters == 4)
 	{
@@ -313,7 +315,7 @@ void tmplput_MAIL_SUMM_SUBJECT(StrBuf *Target, WCTemplputParams *TP)
 }
 int Conditional_MAIL_SUMM_SUBJECT(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 
 
 	return StrLength(Msg->subj) > 0;
@@ -336,13 +338,13 @@ void examine_msgn(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset)
 }
 void tmplput_MAIL_SUMM_INREPLYTO(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	StrBufAppendTemplate(Target, TP, Msg->reply_inreplyto, 0);
 }
 
 int Conditional_MAIL_SUMM_UNREAD(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	return (Msg->Flags & MSGFLAG_READ) != 0;
 }
 
@@ -362,7 +364,7 @@ void examine_wefw(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset)
 }
 void tmplput_MAIL_SUMM_REFIDS(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	StrBufAppendTemplate(Target, TP, Msg->reply_references, 0);
 }
 
@@ -388,7 +390,7 @@ void examine_replyto(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset
 }
 void tmplput_MAIL_SUMM_REPLYTO(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	StrBufAppendTemplate(Target, TP, Msg->ReplyTo, 0);
 }
 
@@ -414,7 +416,7 @@ void examine_cccc(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset)
 }
 void tmplput_MAIL_SUMM_CCCC(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	StrBufAppendTemplate(Target, TP, Msg->cccc, 0);
 }
 
@@ -429,7 +431,7 @@ void examine_room(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset)
 }
 void tmplput_MAIL_SUMM_ORGROOM(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	StrBufAppendTemplate(Target, TP, Msg->Room, 0);
 }
 
@@ -441,22 +443,22 @@ void examine_rfca(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset)
 }
 void tmplput_MAIL_SUMM_RFCA(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	StrBufAppendTemplate(Target, TP, Msg->Rfca, 0);
 }
 int Conditional_MAIL_SUMM_RFCA(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	return StrLength(Msg->Rfca) > 0;
 }
 int Conditional_MAIL_SUMM_CCCC(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	return StrLength(Msg->cccc) > 0;
 }
 int Conditional_MAIL_SUMM_REPLYTO(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	return StrLength(Msg->ReplyTo) > 0;
 }
 
@@ -474,15 +476,30 @@ void examine_node(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset)
 }
 void tmplput_MAIL_SUMM_OTHERNODE(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	StrBufAppendTemplate(Target, TP, Msg->OtherNode, 0);
 }
 int Conditional_MAIL_SUMM_OTHERNODE(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	return StrLength(Msg->OtherNode) > 0;
 }
 
+void examine_nvto(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset)
+{
+	wcsession *WCC = WC;
+
+	CheckConvertBufs(WCC);
+	FreeStrBuf(&Msg->EnvTo);
+	Msg->EnvTo = NewStrBufPlain(NULL, StrLength(HdrLine));
+	StrBuf_RFC822_2_Utf8(Msg->EnvTo, 
+			     HdrLine, 
+			     WCC->DefaultCharset, 
+			     FoundCharset,
+			     WCC->ConvertBuf1,
+			     WCC->ConvertBuf2);
+}
+
 
 void examine_rcpt(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset)
 {
@@ -506,22 +523,22 @@ void examine_rcpt(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset)
 }
 void tmplput_MAIL_SUMM_TO(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	StrBufAppendTemplate(Target, TP, Msg->to, 0);
 }
 int Conditional_MAIL_SUMM_TO(StrBuf *Target, WCTemplputParams *TP) 
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	return StrLength(Msg->to) != 0;
 }
 int Conditional_MAIL_SUMM_SUBJ(StrBuf *Target, WCTemplputParams *TP) 
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	return StrLength(Msg->subj) != 0;
 }
 void tmplput_MAIL_SUMM_ALLRCPT(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	StrBufAppendTemplate(Target, TP, Msg->AllRcpt, 0);
 }
 
@@ -544,34 +561,35 @@ void examine_time(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset)
 void tmplput_MAIL_SUMM_DATE_BRIEF(StrBuf *Target, WCTemplputParams *TP)
 {
 	char datebuf[64];
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	webcit_fmt_date(datebuf, 64, Msg->date, DATEFMT_BRIEF);
 	StrBufAppendBufPlain(Target, datebuf, -1, 0);
 }
 
 void tmplput_MAIL_SUMM_EUID(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	StrBufAppendTemplate(Target, TP, Msg->euid, 0);
 }
 
 void tmplput_MAIL_SUMM_DATE_FULL(StrBuf *Target, WCTemplputParams *TP)
 {
 	char datebuf[64];
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	webcit_fmt_date(datebuf, 64, Msg->date, DATEFMT_FULL);
 	StrBufAppendBufPlain(Target, datebuf, -1, 0);
 }
 void tmplput_MAIL_SUMM_DATE_NO(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	StrBufAppendPrintf(Target, "%ld", Msg->date, 0);
 }
 
 
 
-void render_MAIL(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *FoundCharset)
+void render_MAIL(StrBuf *Target, WCTemplputParams *TP, StrBuf *FoundCharset)
 {
+	wc_mime_attachment *Mime = (wc_mime_attachment *) CTX(CTX_MIME_ATACH);
 	const StrBuf *TemplateMime;
 
 	if (Mime->Data == NULL) 
@@ -592,8 +610,9 @@ void render_MAIL(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *FoundCharset
 */
 }
 
-void render_MIME_VCard(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *FoundCharset)
+void render_MIME_VCard(StrBuf *Target, WCTemplputParams *TP, StrBuf *FoundCharset)
 {
+	wc_mime_attachment *Mime = (wc_mime_attachment *) CTX(CTX_MIME_ATACH);
 	wcsession *WCC = WC;
 	if (StrLength(Mime->Data) == 0)
 		MimeLoadData(Mime);
@@ -618,42 +637,9 @@ void render_MIME_VCard(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *FoundC
 
 }
 
-void render_MIME_VNote(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *FoundCharset)
-{
-	if (StrLength(Mime->Data) == 0)
-		MimeLoadData(Mime);
-	if (StrLength(Mime->Data) > 0) {
-		struct vnote *v;
-		StrBuf *Buf;
-		char *vcard;
-
-		Buf = NewStrBuf();
-		vcard = SmashStrBuf(&Mime->Data);
-		v = vnote_new_from_str(vcard);
-		free (vcard);
-		if (v) {
-			WCTemplputParams TP;
-			
-			memset(&TP, 0, sizeof(WCTemplputParams));
-			TP.Filter.ContextType = CTX_VNOTE;
-			TP.Context = v;
-			DoTemplate(HKEY("mail_vnoteitem"),
-				   Buf, &TP);
-			
-			vnote_free(v);
-			Mime->Data = Buf;
-		}
-		else {
-			if (Mime->Data == NULL)
-				Mime->Data = NewStrBuf();
-			else
-				FlushStrBuf(Mime->Data);
-		}
-	}
-}
-
-void render_MIME_ICS(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *FoundCharset)
+void render_MIME_ICS(StrBuf *Target, WCTemplputParams *TP, StrBuf *FoundCharset)
 {
+	wc_mime_attachment *Mime = (wc_mime_attachment *) CTX(CTX_MIME_ATACH);
 	if (StrLength(Mime->Data) == 0) {
 		MimeLoadData(Mime);
 	}
@@ -744,8 +730,10 @@ void examine_mime_part(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundChars
 }
 
 
-void evaluate_mime_part(message_summary *Msg, wc_mime_attachment *Mime)
+void evaluate_mime_part(StrBuf *Target, WCTemplputParams *TP)
 {
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
+	wc_mime_attachment *Mime = (wc_mime_attachment *) CTX(CTX_MIME_ATACH);
 	void *vMimeRenderer;
 
 	/* just print the root-node */
@@ -786,7 +774,7 @@ void evaluate_mime_part(message_summary *Msg, wc_mime_attachment *Mime)
 
 void tmplput_MAIL_SUMM_NATTACH(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	StrBufAppendPrintf(Target, "%ld", GetCount(Msg->Attachments));
 }
 
@@ -807,12 +795,12 @@ void examine_hnod(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset)
 }
 void tmplput_MAIL_SUMM_H_NODE(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	StrBufAppendTemplate(Target, TP, Msg->hnod, 0);
 }
 int Conditional_MAIL_SUMM_H_NODE(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	return StrLength(Msg->hnod) > 0;
 }
 
@@ -891,14 +879,14 @@ void examine_content_type(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCh
 
 void tmplput_MAIL_SUMM_N(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	StrBufAppendPrintf(Target, "%ld", Msg->msgnum);
 }
 
 
 void tmplput_MAIL_SUMM_PERMALINK(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	char perma_link[1024];
 
 	strcpy(perma_link, "/readfwd?go=");
@@ -910,25 +898,25 @@ void tmplput_MAIL_SUMM_PERMALINK(StrBuf *Target, WCTemplputParams *TP)
 
 int Conditional_MAIL_MIME_ALL(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	return GetCount(Msg->Attachments) > 0;
 }
 
 int Conditional_MAIL_MIME_SUBMESSAGES(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	return GetCount(Msg->Submessages) > 0;
 }
 
 int Conditional_MAIL_MIME_ATTACHLINKS(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	return GetCount(Msg->AttachLinks) > 0;
 }
 
 int Conditional_MAIL_MIME_ATTACH(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	return GetCount(Msg->AllAttach) > 0;
 }
 
@@ -984,28 +972,30 @@ void tmplput_EDIT_WIKI_BODY(StrBuf *Target, WCTemplputParams *TP)
 
 void tmplput_MAIL_BODY(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	StrBufAppendTemplate(Target, TP, Msg->MsgBody->Data, 0);
 }
 
 
-void render_MAIL_variformat(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *FoundCharset)
+void render_MAIL_variformat(StrBuf *Target, WCTemplputParams *TP, StrBuf *FoundCharset)
 {
 	/* Messages in legacy Citadel variformat get handled thusly... */
-	StrBuf *Target = NewStrBufPlain(NULL, StrLength(Mime->Data));
-	FmOut(Target, "JUSTIFY", Mime->Data);
+	wc_mime_attachment *Mime = (wc_mime_attachment *) CTX(CTX_MIME_ATACH);
+	StrBuf *TTarget = NewStrBufPlain(NULL, StrLength(Mime->Data));
+	FmOut(TTarget, "JUSTIFY", Mime->Data);
 	FreeStrBuf(&Mime->Data);
-	Mime->Data = Target;
+	Mime->Data = TTarget;
 }
 
-void render_MAIL_text_plain(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *FoundCharset)
+void render_MAIL_text_plain(StrBuf *Target, WCTemplputParams *TP, StrBuf *FoundCharset)
 {
+	wc_mime_attachment *Mime = (wc_mime_attachment *) CTX(CTX_MIME_ATACH);
 	const char *ptr, *pte;
 	const char *BufPtr = NULL;
 	StrBuf *Line;
 	StrBuf *Line1;
 	StrBuf *Line2;
-	StrBuf *Target;
+	StrBuf *TTarget;
 	long Linecount;
 	long nEmptyLines;
 	int bn = 0;
@@ -1052,7 +1042,7 @@ void render_MAIL_text_plain(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *F
 	Line = NewStrBufPlain(NULL, SIZ);
 	Line1 = NewStrBufPlain(NULL, SIZ);
 	Line2 = NewStrBufPlain(NULL, SIZ);
-	Target = NewStrBufPlain(NULL, StrLength(Mime->Data));
+	TTarget = NewStrBufPlain(NULL, StrLength(Mime->Data));
 	Linecount = 0;
 	nEmptyLines = 0;
 	if (StrLength(Mime->Data) > 0) 
@@ -1079,26 +1069,26 @@ void render_MAIL_text_plain(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *F
 			if (StrLength(Line) == 0) {
 				if (Linecount == 0)
 					continue;
-				StrBufAppendBufPlain(Target, HKEY("<tt></tt><br>\n"), 0);
+				StrBufAppendBufPlain(TTarget, HKEY("<tt></tt><br>\n"), 0);
 
 				nEmptyLines ++;
 				continue;
 			}
 			nEmptyLines = 0;
 			for (i = bn; i < bq; i++)				
-				StrBufAppendBufPlain(Target, HKEY("<blockquote>"), 0);
+				StrBufAppendBufPlain(TTarget, HKEY("<blockquote>"), 0);
 			for (i = bq; i < bn; i++)				
-				StrBufAppendBufPlain(Target, HKEY("</blockquote>"), 0);
+				StrBufAppendBufPlain(TTarget, HKEY("</blockquote>"), 0);
 #ifdef HAVE_ICONV
 			if (ConvertIt) {
 				StrBufConvert(Line, Line1, &ic);
 			}
 #endif
-			StrBufAppendBufPlain(Target, HKEY("<tt>"), 0);
+			StrBufAppendBufPlain(TTarget, HKEY("<tt>"), 0);
 			UrlizeText(Line1, Line, Line2);
 
-			StrEscAppend(Target, Line1, NULL, 0, 0);
-			StrBufAppendBufPlain(Target, HKEY("</tt><br>\n"), 0);
+			StrEscAppend(TTarget, Line1, NULL, 0, 0);
+			StrBufAppendBufPlain(TTarget, HKEY("</tt><br>\n"), 0);
 			bn = bq;
 			Linecount ++;
 		}
@@ -1106,11 +1096,11 @@ void render_MAIL_text_plain(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *F
 	       (BufPtr != NULL));
 
 	if (nEmptyLines > 0)
-		StrBufCutRight(Target, nEmptyLines * (sizeof ("<tt></tt><br>\n") - 1));
+		StrBufCutRight(TTarget, nEmptyLines * (sizeof ("<tt></tt><br>\n") - 1));
 	for (i = 0; i < bn; i++)				
-		StrBufAppendBufPlain(Target, HKEY("</blockquote>"), 0);
+		StrBufAppendBufPlain(TTarget, HKEY("</blockquote>"), 0);
 
-	StrBufAppendBufPlain(Target, HKEY("</i><br>"), 0);
+	StrBufAppendBufPlain(TTarget, HKEY("</i><br>"), 0);
 #ifdef HAVE_ICONV
 	if (ic != (iconv_t)(-1) ) {
 		iconv_close(ic);
@@ -1118,7 +1108,7 @@ void render_MAIL_text_plain(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *F
 #endif
 
 	FreeStrBuf(&Mime->Data);
-	Mime->Data = Target;
+	Mime->Data = TTarget;
 	FlushStrBuf(Mime->ContentType);
 	StrBufAppendBufPlain(Mime->ContentType, HKEY("text/html"), 0);
 	FlushStrBuf(Mime->Charset);
@@ -1128,8 +1118,9 @@ void render_MAIL_text_plain(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *F
 	FreeStrBuf(&Line2);
 }
 
-void render_MAIL_html(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *FoundCharset)
+void render_MAIL_html(StrBuf *Target, WCTemplputParams *TP, StrBuf *FoundCharset)
 {
+	wc_mime_attachment *Mime = (wc_mime_attachment *) CTX(CTX_MIME_ATACH);
 	StrBuf *Buf;
 
 	if (StrLength(Mime->Data) == 0)
@@ -1146,8 +1137,9 @@ void render_MAIL_html(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *FoundCh
 	Mime->Data = Buf;
 }
 
-void render_MAIL_UNKNOWN(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *FoundCharset)
+void render_MAIL_UNKNOWN(StrBuf *Target, WCTemplputParams *TP, StrBuf *FoundCharset)
 {
+	wc_mime_attachment *Mime = (wc_mime_attachment *) CTX(CTX_MIME_ATACH);
 	/* Unknown weirdness */
 	FlushStrBuf(Mime->Data);
 	StrBufAppendBufPlain(Mime->Data, _("I don't know how to display "), -1, 0);
@@ -1158,58 +1150,58 @@ void render_MAIL_UNKNOWN(wc_mime_attachment *Mime, StrBuf *RawData, StrBuf *Foun
 
 HashList *iterate_get_mime_All(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	return Msg->Attachments;
 }
 HashList *iterate_get_mime_Submessages(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	return Msg->Submessages;
 }
 HashList *iterate_get_mime_AttachLinks(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	return Msg->AttachLinks;
 }
 HashList *iterate_get_mime_Attachments(StrBuf *Target, WCTemplputParams *TP)
 {
-	message_summary *Msg = (message_summary*) CTX;
+	message_summary *Msg = (message_summary*) CTX(CTX_MAILSUM);
 	return Msg->AllAttach;
 }
 
 void tmplput_MIME_Name(StrBuf *Target, WCTemplputParams *TP)
 {
-	wc_mime_attachment *mime = (wc_mime_attachment*) CTX;
+	wc_mime_attachment *mime = (wc_mime_attachment*) CTX(CTX_MIME_ATACH);
 	StrBufAppendTemplate(Target, TP, mime->Name, 0);
 }
 
 void tmplput_MIME_FileName(StrBuf *Target, WCTemplputParams *TP)
 {
-	wc_mime_attachment *mime = (wc_mime_attachment*) CTX;
+	wc_mime_attachment *mime = (wc_mime_attachment*) CTX(CTX_MIME_ATACH);
 	StrBufAppendTemplate(Target, TP, mime->FileName, 0);
 }
 
 void tmplput_MIME_PartNum(StrBuf *Target, WCTemplputParams *TP)
 {
-	wc_mime_attachment *mime = (wc_mime_attachment*) CTX;
+	wc_mime_attachment *mime = (wc_mime_attachment*) CTX(CTX_MIME_ATACH);
 	StrBufAppendTemplate(Target, TP, mime->PartNum, 0);
 }
 
 void tmplput_MIME_MsgNum(StrBuf *Target, WCTemplputParams *TP)
 {
-	wc_mime_attachment *mime = (wc_mime_attachment*) CTX;
+	wc_mime_attachment *mime = (wc_mime_attachment*) CTX(CTX_MIME_ATACH);
 	StrBufAppendPrintf(Target, "%ld", mime->msgnum);
 }
 
 void tmplput_MIME_Disposition(StrBuf *Target, WCTemplputParams *TP)
 {
-	wc_mime_attachment *mime = (wc_mime_attachment*) CTX;
+	wc_mime_attachment *mime = (wc_mime_attachment*) CTX(CTX_MIME_ATACH);
 	StrBufAppendTemplate(Target, TP, mime->Disposition, 0);
 }
 
 void tmplput_MIME_ContentType(StrBuf *Target, WCTemplputParams *TP)
 {
-	wc_mime_attachment *mime = (wc_mime_attachment*) CTX;
+	wc_mime_attachment *mime = (wc_mime_attachment*) CTX(CTX_MIME_ATACH);
 	StrBufAppendTemplate(Target, TP, mime->ContentType, 0);
 }
 
@@ -1220,15 +1212,15 @@ void examine_charset(message_summary *Msg, StrBuf *HdrLine, StrBuf *FoundCharset
 
 void tmplput_MIME_Charset(StrBuf *Target, WCTemplputParams *TP)
 {
-	wc_mime_attachment *mime = (wc_mime_attachment*) CTX;
+	wc_mime_attachment *mime = (wc_mime_attachment*) CTX(CTX_MIME_ATACH);
 	StrBufAppendTemplate(Target, TP, mime->Charset, 0);
 }
 
 void tmplput_MIME_Data(StrBuf *Target, WCTemplputParams *TP)
 {
-	wc_mime_attachment *mime = (wc_mime_attachment*) CTX;
+	wc_mime_attachment *mime = (wc_mime_attachment*) CTX(CTX_MIME_ATACH);
 	if (mime->Renderer != NULL)
-		mime->Renderer->f(mime, NULL, NULL);
+		mime->Renderer->f(Target, TP, NULL);
 	StrBufAppendTemplate(Target, TP, mime->Data, 0);
 	/* TODO: check whether we need to load it now? */
 }
@@ -1236,7 +1228,7 @@ void tmplput_MIME_Data(StrBuf *Target, WCTemplputParams *TP)
 void tmplput_MIME_LoadData(StrBuf *Target, WCTemplputParams *TP)
 {
 	wcsession *WCC = WC;	
-	wc_mime_attachment *mime = (wc_mime_attachment*) CTX;
+	wc_mime_attachment *mime = (wc_mime_attachment*) CTX(CTX_MIME_ATACH);
 	wc_mime_attachment *att;
 	
 	if (( (!strcasecmp(ChrPtr(mime->Disposition), "inline"))||
@@ -1264,7 +1256,7 @@ void tmplput_MIME_LoadData(StrBuf *Target, WCTemplputParams *TP)
 
 void tmplput_MIME_Length(StrBuf *Target, WCTemplputParams *TP)
 {
-	wc_mime_attachment *mime = (wc_mime_attachment*) CTX;
+	wc_mime_attachment *mime = (wc_mime_attachment*) CTX(CTX_MIME_ATACH);
 	StrBufAppendPrintf(Target, "%ld", mime->length);
 }
 
@@ -1461,10 +1453,7 @@ int json_RenderView_or_Tail(SharedMessageStatus *Stat,
 			    void **ViewSpecific, 
 			    long oper)
 {
-	WCTemplputParams SubTP;
-
-	memset(&SubTP, 0, sizeof(WCTemplputParams));
-	DoTemplate(HKEY("mailsummary_json"),NULL, &SubTP);
+	DoTemplate(HKEY("mailsummary_json"),NULL, NULL);
 	
 	return 0;
 }
@@ -1484,6 +1473,8 @@ void
 InitModule_MSGRENDERERS
 (void)
 {
+	RegisterCTX(CTX_MAILSUM);
+	RegisterCTX(CTX_MIME_ATACH);
 	RegisterReadLoopHandlerset(
 		VIEW_MAILBOX,
 		mailview_GetParamsGetServerCall,
@@ -1551,22 +1542,22 @@ InitModule_MSGRENDERERS
 	RegisterNamespace("MAIL:QUOTETEXT", 1, 2, tmplput_QUOTED_MAIL_BODY,  NULL, CTX_NONE);
 	RegisterNamespace("MAIL:EDITTEXT", 1, 2, tmplput_EDIT_MAIL_BODY,  NULL, CTX_NONE);
 	RegisterNamespace("MAIL:EDITWIKI", 1, 2, tmplput_EDIT_WIKI_BODY,  NULL, CTX_NONE);
-	RegisterConditional(HKEY("COND:MAIL:SUMM:RFCA"), 0, Conditional_MAIL_SUMM_RFCA,  CTX_MAILSUM);
-	RegisterConditional(HKEY("COND:MAIL:SUMM:CCCC"), 0, Conditional_MAIL_SUMM_CCCC,  CTX_MAILSUM);
-	RegisterConditional(HKEY("COND:MAIL:SUMM:REPLYTO"), 0, Conditional_MAIL_SUMM_REPLYTO,  CTX_MAILSUM);
-	RegisterConditional(HKEY("COND:MAIL:SUMM:UNREAD"), 0, Conditional_MAIL_SUMM_UNREAD, CTX_MAILSUM);
-	RegisterConditional(HKEY("COND:MAIL:SUMM:H_NODE"), 0, Conditional_MAIL_SUMM_H_NODE, CTX_MAILSUM);
-	RegisterConditional(HKEY("COND:MAIL:SUMM:OTHERNODE"), 0, Conditional_MAIL_SUMM_OTHERNODE, CTX_MAILSUM);
-	RegisterConditional(HKEY("COND:MAIL:SUMM:SUBJECT"), 0, Conditional_MAIL_SUMM_SUBJECT, CTX_MAILSUM);
-	RegisterConditional(HKEY("COND:MAIL:ANON"), 0, Conditional_ANONYMOUS_MESSAGE, CTX_MAILSUM);
-	RegisterConditional(HKEY("COND:MAIL:TO"), 0, Conditional_MAIL_SUMM_TO, CTX_MAILSUM);	
-	RegisterConditional(HKEY("COND:MAIL:SUBJ"), 0, Conditional_MAIL_SUMM_SUBJ, CTX_MAILSUM);	
+	RegisterConditional("COND:MAIL:SUMM:RFCA", 0, Conditional_MAIL_SUMM_RFCA,  CTX_MAILSUM);
+	RegisterConditional("COND:MAIL:SUMM:CCCC", 0, Conditional_MAIL_SUMM_CCCC,  CTX_MAILSUM);
+	RegisterConditional("COND:MAIL:SUMM:REPLYTO", 0, Conditional_MAIL_SUMM_REPLYTO,  CTX_MAILSUM);
+	RegisterConditional("COND:MAIL:SUMM:UNREAD", 0, Conditional_MAIL_SUMM_UNREAD, CTX_MAILSUM);
+	RegisterConditional("COND:MAIL:SUMM:H_NODE", 0, Conditional_MAIL_SUMM_H_NODE, CTX_MAILSUM);
+	RegisterConditional("COND:MAIL:SUMM:OTHERNODE", 0, Conditional_MAIL_SUMM_OTHERNODE, CTX_MAILSUM);
+	RegisterConditional("COND:MAIL:SUMM:SUBJECT", 0, Conditional_MAIL_SUMM_SUBJECT, CTX_MAILSUM);
+	RegisterConditional("COND:MAIL:ANON", 0, Conditional_ANONYMOUS_MESSAGE, CTX_MAILSUM);
+	RegisterConditional("COND:MAIL:TO", 0, Conditional_MAIL_SUMM_TO, CTX_MAILSUM);	
+	RegisterConditional("COND:MAIL:SUBJ", 0, Conditional_MAIL_SUMM_SUBJ, CTX_MAILSUM);	
 
 	/* do we have mimetypes to iterate over? */
-	RegisterConditional(HKEY("COND:MAIL:MIME:ATTACH"), 0, Conditional_MAIL_MIME_ALL, CTX_MAILSUM);
-	RegisterConditional(HKEY("COND:MAIL:MIME:ATTACH:SUBMESSAGES"), 0, Conditional_MAIL_MIME_SUBMESSAGES, CTX_MAILSUM);
-	RegisterConditional(HKEY("COND:MAIL:MIME:ATTACH:LINKS"), 0, Conditional_MAIL_MIME_ATTACHLINKS, CTX_MAILSUM);
-	RegisterConditional(HKEY("COND:MAIL:MIME:ATTACH:ATT"), 0, Conditional_MAIL_MIME_ATTACH, CTX_MAILSUM);
+	RegisterConditional("COND:MAIL:MIME:ATTACH", 0, Conditional_MAIL_MIME_ALL, CTX_MAILSUM);
+	RegisterConditional("COND:MAIL:MIME:ATTACH:SUBMESSAGES", 0, Conditional_MAIL_MIME_SUBMESSAGES, CTX_MAILSUM);
+	RegisterConditional("COND:MAIL:MIME:ATTACH:LINKS", 0, Conditional_MAIL_MIME_ATTACHLINKS, CTX_MAILSUM);
+	RegisterConditional("COND:MAIL:MIME:ATTACH:ATT", 0, Conditional_MAIL_MIME_ATTACH, CTX_MAILSUM);
 	RegisterIterator("MAIL:MIME:ATTACH", 0, NULL, iterate_get_mime_All, 
 			 NULL, NULL, CTX_MIME_ATACH, CTX_MAILSUM, IT_NOFLAG);
 	RegisterIterator("MAIL:MIME:ATTACH:SUBMESSAGES", 0, NULL, iterate_get_mime_Submessages, 
@@ -1597,11 +1588,12 @@ InitModule_MSGRENDERERS
 
 	/* mime renderers translate an attachment into webcit viewable html text */
 	RegisterMimeRenderer(HKEY("message/rfc822"), render_MAIL, 0, 150);
-	RegisterMimeRenderer(HKEY("text/vnote"), render_MIME_VNote, 1, 300);
 	RegisterMimeRenderer(HKEY("text/x-vcard"), render_MIME_VCard, 1, 201);
 	RegisterMimeRenderer(HKEY("text/vcard"), render_MIME_VCard, 1, 200);
+//*
 	RegisterMimeRenderer(HKEY("text/calendar"), render_MIME_ICS, 1, 501);
 	RegisterMimeRenderer(HKEY("application/ics"), render_MIME_ICS, 1, 500);
+//*/
 	RegisterMimeRenderer(HKEY("text/x-citadel-variformat"), render_MAIL_variformat, 1, 2);
 	RegisterMimeRenderer(HKEY("text/plain"), render_MAIL_text_plain, 1, 3);
 	RegisterMimeRenderer(HKEY("text"), render_MAIL_text_plain, 1, 1);
@@ -1622,6 +1614,7 @@ InitModule_MSGRENDERERS
 	RegisterMsgHdr(HKEY("rfca"), examine_rfca, 0);
 	RegisterMsgHdr(HKEY("node"), examine_node, 0);
 	RegisterMsgHdr(HKEY("rcpt"), examine_rcpt, 0);
+	RegisterMsgHdr(HKEY("nvto"), examine_nvto, 0);
 	RegisterMsgHdr(HKEY("time"), examine_time, 0);
 	RegisterMsgHdr(HKEY("part"), examine_mime_part, 0);
 	RegisterMsgHdr(HKEY("text"), examine_text, 1);
