@@ -1,7 +1,7 @@
 /*
  * parse urlparts and post data
  *
- * Copyright (c) 1996-2012 by the citadel.org team
+ * Copyright (c) 1996-2013 by the citadel.org team
  *
  * This program is open source software.  You can redistribute it and/or
  * modify it under the terms of the GNU General Public License, version 3.
@@ -15,6 +15,10 @@
 #include "webcit.h"
 #include "webserver.h"
 
+/* uncomment to see all parameters sent to the server by the browser. */
+/* #define DEBUG_URLSTRINGS */
+
+
 void free_url(void *U)
 {
 	urlcontent *u = (urlcontent*) U;
@@ -27,19 +31,21 @@ void free_url(void *U)
  */
 void ParseURLParams(StrBuf *url)
 {
-	const char *aptr, *bptr, *eptr, *up;
-	int len, keylen;
-	urlcontent *u;
+	const char *aptr, *bptr, *eptr, *up = NULL;
+	int len, keylen = 0;
+	urlcontent *u = NULL;
 	wcsession *WCC = WC;
 
-	if (WCC->Hdr->urlstrings == NULL)
+	if (WCC->Hdr->urlstrings == NULL) {
 		WCC->Hdr->urlstrings = NewHash(1, NULL);
+	}
 	eptr = ChrPtr(url) + StrLength(url);
 	up = ChrPtr(url);
 	while ((up < eptr) && (!IsEmptyStr(up))) {
 		aptr = up;
-		while ((aptr < eptr) && (*aptr != '\0') && (*aptr != '='))
+		while ((aptr < eptr) && (*aptr != '\0') && (*aptr != '=')) {
 			aptr++;
+		}
 		if (*aptr != '=') {
 			return;
 		}
@@ -50,16 +56,17 @@ void ParseURLParams(StrBuf *url)
 			bptr++;
 		}
 		keylen = aptr - up - 1; /* -1 -> '=' */
-		if(keylen > sizeof(u->url_key)) {
-			syslog(LOG_WARNING, "invalid url_key");
-			return;
+		if (keylen > sizeof(u->url_key)) {
+			syslog(LOG_WARNING, "%s:%d: invalid url_key of size %d in string size %d",
+				__FILE__, __LINE__, keylen, sizeof(u->url_key)
+			);
 		}
 
 		u = (urlcontent *) malloc(sizeof(urlcontent));
 		memcpy(u->url_key, up, keylen);
 		u->url_key[keylen] = '\0';
 		if (keylen < 0) {
-			syslog(LOG_WARNING, "invalid url_key");
+			syslog(LOG_WARNING, "%s:%d: invalid url_key of size %d", __FILE__, __LINE__, keylen);
 			free(u);
 			return;
 		}
@@ -318,7 +325,7 @@ void upload_handler(char *name, char *filename, char *partnum, char *disp,
 	long keylen;
 
 #ifdef DEBUG_URLSTRINGS
-	syslog(LOG_DEBUG, "upload_handler() name=%s, type=%s, len=%d", name, cbtype, length);
+	syslog(LOG_DEBUG, "upload_handler() name=%s, type=%s, len="SIZE_T_FMT, name, cbtype, length);
 #endif
 	if (WCC->Hdr->urlstrings == NULL)
 		WCC->Hdr->urlstrings = NewHash(1, NULL);
@@ -357,21 +364,19 @@ void upload_handler(char *name, char *filename, char *partnum, char *disp,
 		WCC->upload_filename = NewStrBufPlain(filename, -1);
 		safestrncpy(WCC->upload_content_type, cbtype, sizeof(WC->upload_content_type));
 #ifdef DEBUG_URLSTRINGS
-		syslog(LOG_DEBUG, "File: <%s> len: [%ld]", filename, length);
+		syslog(LOG_DEBUG, "File: <%s> len: [%ld]", filename, (long int)length);
 #endif
 		
 	}
 
 }
 
-
-
 void PutBstr(const char *key, long keylen, StrBuf *Value)
 {
 	urlcontent *u;
 
-	if(keylen > sizeof(u->url_key)) {
-		syslog(LOG_WARNING, "invalid url_key");
+	if(keylen >= sizeof(u->url_key)) {
+		syslog(LOG_WARNING, "%s:%d: invalid url_key of size %ld", __FILE__, __LINE__, keylen);
 		FreeStrBuf(&Value);
 		return;
 	}
@@ -380,6 +385,14 @@ void PutBstr(const char *key, long keylen, StrBuf *Value)
 	u->url_data = Value;
 	Put(WC->Hdr->urlstrings, u->url_key, keylen, u, free_url);
 }
+void PutlBstr(const char *key, long keylen, long Value)
+{
+	StrBuf *Buf;
+
+	Buf = NewStrBufPlain(NULL, sizeof(long) * 16);
+	StrBufPrintf(Buf, "%ld", Value);
+	PutBstr(key, keylen, Buf);
+}
 
 
 
@@ -476,14 +489,88 @@ void tmplput_url_part(StrBuf *Target, WCTemplputParams *TP)
 	}
 }
 
+typedef struct __BstrPair {
+	StrBuf *x;
+	StrBuf *y;
+}BstrPair;
+CtxType CTX_BSTRPAIRS = CTX_NONE;
+void HFreeBstrPair(void *pv)
+{
+	BstrPair *p = (BstrPair*) pv;
+	FreeStrBuf(&p->x);
+	FreeStrBuf(&p->y);
+	free(pv);
+}
+
+HashList *iterate_GetBstrPairs(StrBuf *Target, WCTemplputParams *TP)
+{
+	StrBuf *X, *Y;
+        const char *ch = NULL;
+        long len;
+	const StrBuf *TheBStr;
+	BstrPair *OnePair;
+        HashList *List;
+	const char *Pos = NULL;
+	int i = 0;
+
+	if (HaveTemplateTokenString(NULL, TP, 2, &ch, &len))
+        {
+                GetTemplateTokenString(Target, TP, 2, &ch, &len);
+        }
+	else 
+	{
+		return NULL;
+	}
+
+	TheBStr = SBstr(ch, len);
+	if ((TheBStr == NULL) || (StrLength(TheBStr) == 0))
+		return NULL;
+	List = NewHash(1, NULL);
+	while (Pos != StrBufNOTNULL)
+	{
+		X = NewStrBufPlain(NULL, StrLength(TheBStr));
+		StrBufExtract_NextToken(X, TheBStr, &Pos, '|');
+		if (Pos == StrBufNOTNULL) {
+			FreeStrBuf(&X);
+			DeleteHash(&List);
+			return NULL;
+		}
+		Y = NewStrBufPlain(NULL, StrLength(TheBStr));
+		StrBufExtract_NextToken(Y, TheBStr, &Pos, '|');
+		OnePair = (BstrPair*)malloc(sizeof(BstrPair));
+		OnePair->x = X;
+		OnePair->y = Y;
+		Put(List, IKEY(i), OnePair, HFreeBstrPair);
+		i++;
+	}
+	return List;
+}
+
+
+void tmplput_bstr_pair(StrBuf *Target, WCTemplputParams *TP, int XY)
+{
+	BstrPair *Pair = (BstrPair*) CTX(CTX_BSTRPAIRS);
+
+	StrBufAppendTemplate(Target, TP, (XY)?Pair->y:Pair->x, 0);
+}
+
+void tmplput_bstr_pair_x(StrBuf *Target, WCTemplputParams *TP)
+{	tmplput_bstr_pair(Target, TP, 0); }
+void tmplput_bstr_pair_y(StrBuf *Target, WCTemplputParams *TP)
+{	tmplput_bstr_pair(Target, TP, 1); }
 
 void 
 InitModule_PARAMHANDLING
 (void)
 {
+	RegisterCTX(CTX_BSTRPAIRS);
 	WebcitAddUrlHandler(HKEY("diagnostics"), "", 0, diagnostics, NEED_URL);
 
-	RegisterConditional(HKEY("COND:BSTR"), 1, ConditionalBstr, CTX_NONE);
+	RegisterIterator("ITERATE:BSTR:PAIR", 1, NULL, iterate_GetBstrPairs, NULL, DeleteHash, CTX_BSTRPAIRS, CTX_NONE, IT_NOFLAG);
+	RegisterNamespace("BSTR:PAIR:X", 1, 2, tmplput_bstr_pair_x, NULL, CTX_BSTRPAIRS);
+	RegisterNamespace("BSTR:PAIR:Y", 1, 2, tmplput_bstr_pair_y, NULL, CTX_BSTRPAIRS);
+
+	RegisterConditional("COND:BSTR", 1, ConditionalBstr, CTX_NONE);
 	RegisterNamespace("BSTR", 1, 2, tmplput_bstr, NULL, CTX_NONE);
 	RegisterNamespace("BSTR:FORWARD", 1, 2, tmplput_bstrforward, NULL, CTX_NONE);
 	RegisterNamespace("URLPART", 1, 2, tmplput_url_part, NULL, CTX_NONE);
