@@ -43,6 +43,7 @@ const char EmptyStr[]="";
 #define SV_PREEVALUATED 6
 
 
+
 /*
  * Dynamic content for variable substitution in templates
  */
@@ -71,8 +72,13 @@ typedef struct _HashHandler {
 	WCHandlerFunc HandlerFunc;
 }HashHandler;
 
+typedef enum _estate {
+	eNext,
+	eSkipTilEnd
+} TemplState;
+
 void *load_template(StrBuf *Target, WCTemplate *NewTemplate);
-int EvaluateConditional(StrBuf *Target, int Neg, int state, WCTemplputParams *TP);
+int EvaluateConditional(StrBuf *Target, int Neg, int state, WCTemplputParams **TPP);
 
 
 
@@ -83,41 +89,124 @@ typedef struct _SortStruct {
 	CompareFunc Reverse;
 	CompareFunc GroupChange;
 
-	long ContextType;
+	CtxType ContextType;
 }SortStruct;
 
-const char *CtxNames[]  = {
-	"Context NONE",
-	"Context SITECFG",
-	"Context SESSION",
-	"Context INETCFG",
-	"Context VNOTE",
-	"Context WHO",
-	"Context PREF",
-	"Context NODECONF",
-	"Context USERLIST",
-	"Context MAILSUM",
-	"Context MIME_ATACH",
-	"Context FILELIST",
-	"Context STRBUF",
-	"Context STRBUFARR",
-	"Context LONGVECTOR",
-	"Context ROOMS",
-	"Context FLOORS",
-	"Context ITERATE",
-	"Context ICAL",
-	"Context DavNamespace",
-	"Context TAB",
-	"Context VCARD",
-	"Context SIEVE List",
-	"Context SIEVE Script",
-	"Context MailQ-Item",
-	"Context MailQ-Recipient",
-	"Context ServLogStatus",
-	"Context UNKNOWN"
-};
+HashList *CtxList = NULL;
 
+static CtxType CtxCounter = CTX_NONE;
 
+CtxType CTX_STRBUF = CTX_NONE;
+CtxType CTX_STRBUFARR = CTX_NONE;
+CtxType CTX_LONGVECTOR = CTX_NONE;
+
+CtxType CTX_ITERATE = CTX_NONE;
+CtxType CTX_TAB = CTX_NONE;
+
+void HFreeContextType(void *pCtx)
+{
+	CtxTypeStruct *FreeStruct = (CtxTypeStruct *) pCtx;
+	FreeStrBuf(&FreeStruct->Name);
+	free(FreeStruct);
+}
+void PutContextType(const char *name, long len, CtxType TheCtx)
+{
+	CtxTypeStruct *NewStruct;
+
+	NewStruct = (CtxTypeStruct*) malloc(sizeof(CtxTypeStruct));
+	NewStruct->Name = NewStrBufPlain(name, len);
+	NewStruct->Type = TheCtx;
+
+	Put(CtxList, IKEY(NewStruct->Type), NewStruct, HFreeContextType);
+}
+void RegisterContextType(const char *name, long len, CtxType *TheCtx)
+{
+	if (*TheCtx != CTX_NONE)
+		return;
+
+	*TheCtx = ++CtxCounter;
+	PutContextType(name, len, *TheCtx);
+}
+
+CtxTypeStruct *GetContextType(CtxType Type)
+{
+	void *pv = NULL;
+	GetHash(CtxList, IKEY(Type), &pv);
+	return pv;
+}
+
+const char *UnknownContext = "CTX_UNKNOWN";
+
+const char *ContextName(CtxType ContextType)
+{
+	CtxTypeStruct *pCtx;
+
+	pCtx = GetContextType(ContextType);
+
+	if (pCtx != NULL) 
+		return ChrPtr(pCtx->Name);
+	else
+		return UnknownContext;
+}
+
+void StackDynamicContext(WCTemplputParams *Super, 
+			 WCTemplputParams *Sub, 
+			 void *Context,
+			 CtxType ContextType,
+			 int nArgs,
+			 WCTemplateToken *Tokens, 
+			 WCConditionalFunc ExitCtx, 
+			 long ExitCTXID)
+{
+	memset(Sub, 0, sizeof(WCTemplputParams));
+
+	if (Super != NULL) {
+		Sub->Sub = Super->Sub;
+		Super->Sub = Sub;
+	}
+	if (Sub->Sub != NULL)
+		Sub->Sub->Super = Sub;
+	Sub->Super = Super;
+	
+	Sub->Context = Context;
+	Sub->Filter.ContextType = ContextType;
+	Sub->nArgs = nArgs;
+	Sub->Tokens = Tokens;
+	Sub->ExitCtx = ExitCtx;
+	Sub->ExitCTXID = ExitCTXID;
+}
+
+void UnStackContext(WCTemplputParams *Sub)
+{
+	if (Sub->Super != NULL)
+	{
+		Sub->Super->Sub = Sub->Sub;
+	}
+	if (Sub->Sub != NULL)
+	{
+		Sub->Sub->Super = Sub->Super;
+	}
+}
+void UnStackDynamicContext(StrBuf *Target, WCTemplputParams **TPP)
+{
+	WCTemplputParams *TP = *TPP;
+	WCTemplputParams *Super = TP->Super;
+	TP->ExitCtx(Target, TP);
+	*TPP = Super;
+}
+
+void *GetContextPayload(WCTemplputParams *TP, CtxType ContextType)
+{
+	WCTemplputParams *whichTP = TP;
+
+	if (ContextType == CTX_NONE)
+		return TP->Context;
+
+	while ((whichTP != NULL) && (whichTP->Filter.ContextType != ContextType))
+		whichTP = whichTP->Super;
+
+	return whichTP->Context;	
+}
 
 void DestroySortStruct(void *vSort)
 {
@@ -127,13 +216,6 @@ void DestroySortStruct(void *vSort)
 	free (Sort);
 }
 
-const char *ContextName(int ContextType)
-{
-	if (ContextType < CTX_UNKNOWN)
-		return CtxNames[ContextType];
-	else
-		return CtxNames[CTX_UNKNOWN];
-}
 
 void LogTemplateError (StrBuf *Target, const char *Type, int ErrorPos, WCTemplputParams *TP, const char *Format, ...)
 {
@@ -176,10 +258,6 @@ void LogTemplateError (StrBuf *Target, const char *Type, int ErrorPos, WCTemplpu
 		       Type, 
 		       ChrPtr(Error));
 	}
-/*
-	if (Target == NULL) 
-		return;
-*/
 	WCC = WC;
 	if (WCC == NULL) {
 		FreeStrBuf(&Info);
@@ -244,9 +322,6 @@ void LogTemplateError (StrBuf *Target, const char *Type, int ErrorPos, WCTemplpu
 */
 }
 
-
-
-
 void LogError (StrBuf *Target, const char *Type, const char *Format, ...)
 {
 	wcsession *WCC;
@@ -286,7 +361,7 @@ void RegisterNS(const char *NSName,
 		int nMaxArgs, 
 		WCHandlerFunc HandlerFunc, 
 		WCPreevalFunc PreevalFunc,
-		int ContextRequired)
+		CtxType ContextRequired)
 {
 	HashHandler *NewHandler;
 	
@@ -295,55 +370,38 @@ void RegisterNS(const char *NSName,
 	NewHandler->Filter.nMinArgs = nMinArgs;
 	NewHandler->Filter.nMaxArgs = nMaxArgs;
 	NewHandler->Filter.ContextType = ContextRequired;
-	NewHandler->Filter.ControlContextType = CTX_NONE;
 
 	NewHandler->PreEvalFunc = PreevalFunc;
 	NewHandler->HandlerFunc = HandlerFunc;	
 	Put(GlobalNS, NSName, len, NewHandler, NULL);
 }
 
-void RegisterControlNS(const char *NSName, 
-		       long len, 
-		       int nMinArgs, 
-		       int nMaxArgs, 
-		       WCHandlerFunc HandlerFunc, 
-		       int ControlContextRequired)
-{
-	HashHandler *NewHandler;
-	
-	NewHandler = (HashHandler*) malloc(sizeof(HashHandler));
-	memset(NewHandler, 0, sizeof(HashHandler));
-	NewHandler->Filter.nMinArgs = nMinArgs;
-	NewHandler->Filter.nMaxArgs = nMaxArgs;
-	NewHandler->Filter.ContextType = CTX_NONE;
-	NewHandler->Filter.ControlContextType = ControlContextRequired;
-	NewHandler->HandlerFunc = HandlerFunc;	
-	Put(GlobalNS, NSName, len, NewHandler, NULL);
-}
-
 
 
 int CheckContext(StrBuf *Target, ContextFilter *Need, WCTemplputParams *TP, const char *ErrType)
 {
-	if ((Need->ContextType != CTX_NONE) && 
-	    (Need->ContextType != TP->Filter.ContextType)) {
+	WCTemplputParams *TPP = TP;
+	
+	if ((Need != NULL) &&
+	    (Need->ContextType != CTX_NONE) && 
+	    (Need->ContextType != TPP->Filter.ContextType)) {
+
+		while ((TPP != NULL) && 
+		       (Need->ContextType != TPP->Filter.ContextType))
+		{
+			TPP = TPP->Super;
+		}
+
+		if (TPP != NULL)
+			return 1;
+
                 LogTemplateError(
-                        Target, ErrType, ERR_PARM1, TP,
+                        Target, ErrType, ERR_NAME, TP,
 			"  WARNING: requires Context: [%s], have [%s]!", 
 			ContextName(Need->ContextType), 
 			ContextName(TP->Filter.ContextType));
 		return 0;
 	}
-
-	if ((Need->ControlContextType != CTX_NONE) && 
-	    (Need->ControlContextType != TP->Filter.ControlContextType)) {
-                LogTemplateError(
-                        Target, ErrType, ERR_PARM1, TP,
-			"  WARNING: requires Control Context: [%s], have [%s]!", 
-			ContextName(Need->ControlContextType), 
-			ContextName(TP->Filter.ControlContextType));
-		return 0;
-	}
 /*			
 	if (TP->Tokens->nParameters < Need->nMinArgs) {
 		LogTemplateError(Target, ErrType, ERR_NAME, TP,
@@ -410,6 +468,7 @@ int HaveTemplateTokenString(StrBuf *Target,
 	case TYPE_STR:
 	case TYPE_BSTR:
 	case TYPE_PREFSTR:
+	case TYPE_ROOMPREFSTR:
 	case TYPE_GETTEXT:
 	case TYPE_SUBTEMPLATE:
 		return 1;
@@ -427,7 +486,6 @@ void GetTemplateTokenString(StrBuf *Target,
 			    long *len)
 {
 	StrBuf *Buf;
-	WCTemplputParams SubTP;
 
 	if (N >= TP->Tokens->nParameters) {
 		LogTemplateError(Target, 
@@ -471,6 +529,19 @@ void GetTemplateTokenString(StrBuf *Target,
 		*Value = ChrPtr(Buf);
 		*len = StrLength(Buf);
 		break;
+	case TYPE_ROOMPREFSTR:
+		if (TP->Tokens->Params[N]->len == 0) {
+			LogTemplateError(Target, 
+					 "TokenParameter", N, TP, 
+					 "Requesting parameter %d; of type PREFSTR, empty lookup string not admitted.", N);
+			*len = 0;
+			*Value = EmptyStr;
+			break;
+		}
+		Buf = get_ROOM_PREFS(TKEY(N));
+		*Value = ChrPtr(Buf);
+		*len = StrLength(Buf);
+		break;
 	case TYPE_LONG:
 		LogTemplateError(Target, 
 				 "TokenParameter", N, TP, 
@@ -495,11 +566,9 @@ void GetTemplateTokenString(StrBuf *Target,
 			break;
 		}
 
-		memset(&SubTP, 0, sizeof(WCTemplputParams *));
-		SubTP.Context = TP->Context;
-		SubTP.Filter.ContextType = TP->Filter.ContextType;
 		Buf = NewStrBuf();
-		DoTemplate(TKEY(N), Buf, &SubTP);
+		DoTemplate(TKEY(N), Buf, TP);
+
 		*Value = ChrPtr(Buf);
 		*len = StrLength(Buf);
 		/* we can't free it here, so we put it into the subst so its discarded later on. */
@@ -552,6 +621,19 @@ long GetTemplateTokenNumber(StrBuf *Target, WCTemplputParams *TP, int N, long df
 		if (get_PREF_LONG(TKEY(N), &Ret, dflt))
 			return Ret;
 		return 0;
+	case TYPE_ROOMPREFSTR:
+		LogTemplateError(Target, 
+				 "TokenParameter", N, TP, 
+				 "requesting a prefstring in param %d want a number", N);
+		if (TP->Tokens->Params[N]->len == 0) {
+			LogTemplateError(Target, 
+					 "TokenParameter", N, TP, 
+					 "Requesting parameter %d; of type PREFSTR, empty lookup string not admitted.", N);
+			return 0;
+		}
+		if (get_ROOM_PREFS_LONG(TKEY(N), &Ret, dflt))
+			return Ret;
+		return 0;
 	case TYPE_INTDEFINE:
 	case TYPE_LONG:
 		return TP->Tokens->Params[N]->lvalue;
@@ -631,6 +713,55 @@ void StrBufAppendTemplate(StrBuf *Target,
 	}
 }
 
+/*
+ * puts string into the template and computes which escape methon we should use
+ * Source = the string we should put into the template
+ * FormatTypeIndex = where should we look for escape types if?
+ */
+void StrBufAppendTemplateStr(StrBuf *Target, 
+			     WCTemplputParams *TP,
+			     const char *Source, int FormatTypeIndex)
+{
+	const char *pFmt = NULL;
+	char EscapeAs = ' ';
+
+	if ((FormatTypeIndex < TP->Tokens->nParameters) &&
+	    (TP->Tokens->Params[FormatTypeIndex]->Type == TYPE_STR) &&
+	    (TP->Tokens->Params[FormatTypeIndex]->len >= 1)) {
+		pFmt = TP->Tokens->Params[FormatTypeIndex]->Start;
+		EscapeAs = *pFmt;
+	}
+
+	switch(EscapeAs)
+	{
+	case 'H':
+		StrEscAppend(Target, NULL, Source, 0, 2);
+		break;
+	case 'X':
+		StrEscAppend(Target, NULL, Source, 0, 0);
+		break;
+	case 'J':
+		StrECMAEscAppend(Target, NULL, Source);
+	  break;
+	case 'K':
+		StrHtmlEcmaEscAppend(Target, NULL, Source, 0, 0);
+	  break;
+	case 'U':
+		StrBufUrlescAppend(Target, NULL, Source);
+		break;
+/*
+	case 'F':
+		if (pFmt != NULL) 	pFmt++;
+		else			pFmt = "JUSTIFY";
+		if (*pFmt == '\0')	pFmt = "JUSTIFY";
+		FmOut(Target, pFmt, Source);
+		break;
+*/
+	default:
+		StrBufAppendBufPlain(Target, Source, -1, 0);
+	}
+}
+
 
 void PutNewToken(WCTemplate *Template, WCTemplateToken *NewToken)
 {
@@ -694,6 +825,14 @@ int GetNextParameter(StrBuf *Buf,
 			ParamBrace = 1;
 		}
 	}
+	else if (*pch == '.') {
+		Parm->Type = TYPE_ROOMPREFSTR;
+		pch ++;
+		if (*pch == '(') {
+			pch ++;
+			ParamBrace = 1;
+		}
+	}
 	else if (*pch == ';') {
 		Parm->Type = TYPE_PREFINT;
 		pch ++;
@@ -762,7 +901,7 @@ int GetNextParameter(StrBuf *Buf,
 			StrBufPeek(Buf, pch, -1, '\0');		
 			if (LoadTemplates > 1) {			
 				syslog(LOG_DEBUG,
-					"DBG: got param [%s] %d %d\n", 
+					"DBG: got param [%s] "SIZE_T_FMT" "SIZE_T_FMT"\n", 
 					pchs, pche - pchs, strlen(pchs)
 				);
 			}
@@ -1060,15 +1199,14 @@ WCTemplateToken *NewTemplateSubstitute(StrBuf *Buf,
 		} else {
 			LogTemplateError(
 				NULL, "Token ", ERR_NAME, &TP,
-				" isn't known to us.", 
-				NULL);
+				" isn't known to us.");
 		}
 		break;
 	case SV_GETTEXT:
-		if (NewToken->nParameters !=1) {
+		if ((NewToken->nParameters < 1) || (NewToken->nParameters > 2)) {
 			LogTemplateError(                               
 				NULL, "Gettext", ERR_NAME, &TP,
-				"requires exactly 1 parameter, you gave %d params", 
+				"requires 1 or 2 parameter, you gave %d params", 
 				NewToken->nParameters);
 			NewToken->Flags = 0;
 			break;
@@ -1387,8 +1525,13 @@ int LoadTemplateDir(const StrBuf *DirName, HashList *big, const StrBuf *BaseKey)
 	{
 		char *MinorPtr;
 
-#ifdef _DIRENT_HAVE_D_NAMELEN
-		d_namelen = filedir_entry->d_namelen;
+#ifdef _DIRENT_HAVE_D_NAMLEN
+		d_namelen = filedir_entry->d_namlen;
+#else
+		d_namelen = strlen(filedir_entry->d_name);
+#endif
+
+#ifdef _DIRENT_HAVE_D_TYPE
 		d_type = filedir_entry->d_type;
 #else
 
@@ -1401,7 +1544,6 @@ int LoadTemplateDir(const StrBuf *DirName, HashList *big, const StrBuf *BaseKey)
 #define IFTODT(mode)   (((mode) & 0170000) >> 12)
 #define DTTOIF(dirtype)        ((dirtype) << 12)
 #endif
-		d_namelen = strlen(filedir_entry->d_name);
 		d_type = DT_UNKNOWN;
 #endif
 		d_without_ext = d_namelen;
@@ -1423,7 +1565,7 @@ int LoadTemplateDir(const StrBuf *DirName, HashList *big, const StrBuf *BaseKey)
 			char path[PATH_MAX];
 			snprintf(path, PATH_MAX, "%s/%s", 
 				 ChrPtr(DirName), filedir_entry->d_name);
-			if (stat(path, &s) == 0) {
+			if (lstat(path, &s) == 0) {
 				d_type = IFTODT(s.st_mode);
 			}
 		}
@@ -1451,7 +1593,7 @@ int LoadTemplateDir(const StrBuf *DirName, HashList *big, const StrBuf *BaseKey)
 			LoadTemplateDir(SubDirectory, big, SubKey);
 
 			break;
-		case DT_LNK: /* TODO: check whether its a file or a directory */
+		case DT_LNK: 
 		case DT_REG:
 
 
@@ -1562,12 +1704,13 @@ void InitTemplateCache(void)
  * \param state are we in conditional state?
  * \param ContextType what type of information does context giv us?
  */
-int EvaluateToken(StrBuf *Target, int state, WCTemplputParams *TP)
+int EvaluateToken(StrBuf *Target, int state, WCTemplputParams **TPP)
 {
 	const char *AppendMe;
 	long AppendMeLen;
 	HashHandler *Handler;
 	void *vVar;
+	WCTemplputParams *TP = *TPP;
 	
 /* much output, since pName is not terminated...
 	syslog(LOG_DEBUG,"Doing token: %s\n",Token->pName);
@@ -1578,14 +1721,26 @@ int EvaluateToken(StrBuf *Target, int state, WCTemplputParams *TP)
 		TmplGettext(Target, TP);
 		break;
 	case SV_CONDITIONAL: /** Forward conditional evaluation */
-		return EvaluateConditional(Target, 1, state, TP);
+		Handler = (HashHandler*) TP->Tokens->PreEval;
+		if (!CheckContext(Target, &Handler->Filter, TP, "Conditional")) {
+			return 0;
+		}
+		return EvaluateConditional(Target, 1, state, TPP);
 		break;
 	case SV_NEG_CONDITIONAL: /** Reverse conditional evaluation */
-		return EvaluateConditional(Target, 0, state, TP);
+		Handler = (HashHandler*) TP->Tokens->PreEval;
+		if (!CheckContext(Target, &Handler->Filter, TP, "Conditional")) {
+			return 0;
+		}
+		return EvaluateConditional(Target, 0, state, TPP);
 		break;
 	case SV_CUST_STR_CONDITIONAL: /** Conditional put custom strings from params */
+		Handler = (HashHandler*) TP->Tokens->PreEval;
+		if (!CheckContext(Target, &Handler->Filter, TP, "Conditional")) {
+			return 0;
+		}
 		if (TP->Tokens->nParameters >= 6) {
-			if (EvaluateConditional(Target, 0, state, TP)) {
+			if (EvaluateConditional(Target, 0, state, TPP)) {
 				GetTemplateTokenString(Target, TP, 5, &AppendMe, &AppendMeLen);
 				StrBufAppendBufPlain(Target, 
 						     AppendMe, 
@@ -1599,6 +1754,10 @@ int EvaluateToken(StrBuf *Target, int state, WCTemplputParams *TP)
 						     AppendMeLen,
 						     0);
 			}
+			if (*TPP != TP)
+			{
+				UnStackDynamicContext(Target, TPP);
+			}
 		}
 		else  {
 			LogTemplateError(
@@ -1613,7 +1772,7 @@ int EvaluateToken(StrBuf *Target, int state, WCTemplputParams *TP)
 	case SV_PREEVALUATED:
 		Handler = (HashHandler*) TP->Tokens->PreEval;
 		if (!CheckContext(Target, &Handler->Filter, TP, "Token")) {
-			return -1;
+			return 0;
 		}
 		Handler->HandlerFunc(Target, TP);
 		break;		
@@ -1621,7 +1780,7 @@ int EvaluateToken(StrBuf *Target, int state, WCTemplputParams *TP)
 		if (GetHash(GlobalNS, TP->Tokens->pName, TP->Tokens->NameEnd, &vVar)) {
 			Handler = (HashHandler*) vVar;
 			if (!CheckContext(Target, &Handler->Filter, TP, "Token")) {
-				return -1;
+				return 0;
 			}
 			else {
 				Handler->HandlerFunc(Target, TP);
@@ -1642,15 +1801,20 @@ const StrBuf *ProcessTemplate(WCTemplate *Tmpl, StrBuf *Target, WCTemplputParams
 {
 	WCTemplate *pTmpl = Tmpl;
 	int done = 0;
-	int i, state;
+	int i;
+	TemplState state;
 	const char *pData, *pS;
 	long len;
 	WCTemplputParams TP;
+	WCTemplputParams *TPtr = &TP;
+
+	memset(TPtr, 0, sizeof(WCTemplputParams));
 
 	memcpy(&TP.Filter, &CallingTP->Filter, sizeof(ContextFilter));
 
 	TP.Context = CallingTP->Context;
-	TP.ControlContext = CallingTP->ControlContext;
+	TP.Sub = CallingTP->Sub;
+	TP.Super = CallingTP->Super;
 
 	if (LoadTemplates != 0) {			
 		if (LoadTemplates > 1)
@@ -1671,7 +1835,7 @@ const StrBuf *ProcessTemplate(WCTemplate *Tmpl, StrBuf *Target, WCTemplputParams
 	pS = pData = ChrPtr(pTmpl->Data);
 	len = StrLength(pTmpl->Data);
 	i = 0;
-	state = 0;
+	state = eNext;
 	while (!done) {
 		if (i >= pTmpl->nTokensUsed) {
 			StrBufAppendBufPlain(Target, 
@@ -1680,28 +1844,56 @@ const StrBuf *ProcessTemplate(WCTemplate *Tmpl, StrBuf *Target, WCTemplputParams
 			done = 1;
 		}
 		else {
+			int TokenRc = 0;
+
 			StrBufAppendBufPlain(
 				Target, pData, 
 				pTmpl->Tokens[i]->pTokenStart - pData, 0);
-			TP.Tokens = pTmpl->Tokens[i];
-			TP.nArgs = pTmpl->Tokens[i]->nParameters;
-			state = EvaluateToken(Target, state, &TP);
+			TPtr->Tokens = pTmpl->Tokens[i];
+			TPtr->nArgs = pTmpl->Tokens[i]->nParameters;
 
-			while ((state != 0) && (i+1 < pTmpl->nTokensUsed)) {
+		        TokenRc = EvaluateToken(Target, TokenRc, &TPtr);
+			if (TokenRc > 0)
+			{
+				state = eSkipTilEnd;
+			}
+			else if (TokenRc < 0)
+			{
+				if ((TPtr != &TP) &&
+				    (TPtr->ExitCTXID == -TokenRc))
+				{
+					UnStackDynamicContext(Target, &TPtr);
+				}
+				TokenRc = 0;
+			}
+
+			while ((state != eNext) && (i+1 < pTmpl->nTokensUsed)) {
 			/* condition told us to skip till its end condition */
 				i++;
-				TP.Tokens = pTmpl->Tokens[i];
-				TP.nArgs = pTmpl->Tokens[i]->nParameters;
+				TPtr->Tokens = pTmpl->Tokens[i];
+				TPtr->nArgs = pTmpl->Tokens[i]->nParameters;
 				if ((pTmpl->Tokens[i]->Flags == SV_CONDITIONAL) ||
-				    (pTmpl->Tokens[i]->Flags == SV_NEG_CONDITIONAL)) {
-					if (state == EvaluateConditional(
-						    Target, 
-						    pTmpl->Tokens[i]->Flags, 
-						    state, 
-						    &TP))
-						state = 0;
+				    (pTmpl->Tokens[i]->Flags == SV_NEG_CONDITIONAL))
+				{
+					int rc;
+				        rc = EvaluateConditional(
+						Target, 
+						pTmpl->Tokens[i]->Flags, 
+						TokenRc, 
+						&TPtr);
+					if (-rc == TokenRc)
+					{
+						TokenRc = 0;
+						state = eNext;
+						if ((TPtr != &TP) &&
+						    (TPtr->ExitCTXID == - rc))
+						{
+							UnStackDynamicContext(Target, &TPtr);
+						}
+					}
 				}
 			}
+
 			pData = pTmpl->Tokens[i++]->pTokenEnd + 1;
 			if (i > pTmpl->nTokensUsed)
 				done = 1;
@@ -1785,8 +1977,8 @@ void tmplput_Comment(StrBuf *Target, WCTemplputParams *TP)
 typedef struct _HashIterator {
 	HashList *StaticList;
 	int AdditionalParams;
-	int ContextType;
-	int XPectContextType;
+	CtxType ContextType;
+	CtxType XPectContextType;
 	int Flags;
 	RetrieveHashlistFunc GetHash;
 	HashDestructorFunc Destructor;
@@ -1799,8 +1991,8 @@ void RegisterITERATOR(const char *Name, long len,
 		      RetrieveHashlistFunc GetHash, 
 		      SubTemplFunc DoSubTempl,
 		      HashDestructorFunc Destructor,
-		      int ContextType, 
-		      int XPectContextType, 
+		      CtxType ContextType, 
+		      CtxType XPectContextType, 
 		      int Flags)
 {
 	HashIterator *It;
@@ -1882,6 +2074,7 @@ void tmpl_iterate_subtmpl(StrBuf *Target, WCTemplputParams *TP)
 	void *vContext;
 	void *vLastContext = NULL;
 	StrBuf *SubBuf;
+	WCTemplputParams IterateTP;
 	WCTemplputParams SubTP;
 	IterateStruct Status;
 
@@ -1890,7 +2083,6 @@ void tmpl_iterate_subtmpl(StrBuf *Target, WCTemplputParams *TP)
 	long StopAt = -1;
 
 	memset(&Status, 0, sizeof(IterateStruct));
-	memcpy (&SubTP, &TP, sizeof(WCTemplputParams));
 	
 	It = (HashIterator*) TP->Tokens->Preeval2;
 	if (It == NULL) {
@@ -1945,42 +2137,47 @@ void tmpl_iterate_subtmpl(StrBuf *Target, WCTemplputParams *TP)
 		}
 	}
 	nMembersUsed = GetCount(List);
-	SubBuf = NewStrBuf();
-	SubTP.Filter.ContextType = It->ContextType;
-	SubTP.Filter.ControlContextType = CTX_ITERATE;
-	SubTP.ControlContext = &Status;
+
+	StackContext (TP, &IterateTP, &Status, CTX_ITERATE, 0, TP->Tokens);
+	{
+		SubBuf = NewStrBuf();
 	
-	if (HAVE_PARAM(2)) {
-		StartAt = GetTemplateTokenNumber(Target, TP, 2, 0);
-	}
-	if (HAVE_PARAM(3)) {
-		StepWidth = GetTemplateTokenNumber(Target, TP, 3, 0);
-	}
-	if (HAVE_PARAM(4)) {
-		StopAt = GetTemplateTokenNumber(Target, TP, 4, -1);
-	}
-	it = GetNewHashPos(List, StepWidth);
-	if (StopAt < 0) {
-		StopAt = GetCount(List);
-	}
-	while (GetNextHashPos(List, it, &Status.KeyLen, &Status.Key, &vContext)) {
-		if ((Status.n >= StartAt) && (Status.n <= StopAt)) {
-			if (DetectGroupChange && Status.n > 0) {
-				Status.GroupChange = SortBy->GroupChange(vContext, vLastContext);
+		if (HAVE_PARAM(2)) {
+			StartAt = GetTemplateTokenNumber(Target, TP, 2, 0);
+		}
+		if (HAVE_PARAM(3)) {
+			StepWidth = GetTemplateTokenNumber(Target, TP, 3, 0);
+		}
+		if (HAVE_PARAM(4)) {
+			StopAt = GetTemplateTokenNumber(Target, TP, 4, -1);
+		}
+		it = GetNewHashPos(List, StepWidth);
+		if (StopAt < 0) {
+			StopAt = GetCount(List);
+		}
+		while (GetNextHashPos(List, it, &Status.KeyLen, &Status.Key, &vContext)) {
+			if ((Status.n >= StartAt) && (Status.n <= StopAt)) {
+				if (DetectGroupChange && Status.n > 0) {
+					Status.GroupChange = SortBy->GroupChange(vContext, vLastContext);
+				}
+				Status.LastN = (Status.n + 1) == nMembersUsed;
+				StackContext(&IterateTP, &SubTP, vContext, It->ContextType, 0, NULL);
+				{
+					if (It->DoSubTemplate != NULL)
+						It->DoSubTemplate(SubBuf, &SubTP);
+					DoTemplate(TKEY(1), SubBuf, &SubTP);
+
+					StrBufAppendBuf(Target, SubBuf, 0);
+					FlushStrBuf(SubBuf);
+				}
+				UnStackContext(&SubTP);
+				Status.oddeven = ! Status.oddeven;
+				vLastContext = vContext;
 			}
-			Status.LastN = (Status.n + 1) == nMembersUsed;
-			SubTP.Context = vContext;
-			if (It->DoSubTemplate != NULL)
-				It->DoSubTemplate(SubBuf, &SubTP);
-			DoTemplate(TKEY(1), SubBuf, &SubTP);
-			
-			StrBufAppendBuf(Target, SubBuf, 0);
-			FlushStrBuf(SubBuf);
-			Status.oddeven = ! Status.oddeven;
-			vLastContext = vContext;
+			Status.n++;
 		}
-		Status.n++;
 	}
+	UnStackContext(&IterateTP);
 	FreeStrBuf(&SubBuf);
 	DeleteHashPos(&it);
 	if (It->Destructor != NULL)
@@ -1990,7 +2187,7 @@ void tmpl_iterate_subtmpl(StrBuf *Target, WCTemplputParams *TP)
 
 int conditional_ITERATE_ISGROUPCHANGE(StrBuf *Target, WCTemplputParams *TP)
 {
-	IterateStruct *Ctx = CCTX;
+	IterateStruct *Ctx = CTX(CTX_ITERATE);
 	if (TP->Tokens->nParameters < 3)
 		return 	Ctx->GroupChange;
 
@@ -1999,7 +2196,7 @@ int conditional_ITERATE_ISGROUPCHANGE(StrBuf *Target, WCTemplputParams *TP)
 
 void tmplput_ITERATE_ODDEVEN(StrBuf *Target, WCTemplputParams *TP)
 {
-	IterateStruct *Ctx = CCTX;
+	IterateStruct *Ctx = CTX(CTX_ITERATE);
 	if (Ctx->oddeven)
 		StrBufAppendBufPlain(Target, HKEY("odd"), 0);
 	else
@@ -2009,7 +2206,7 @@ void tmplput_ITERATE_ODDEVEN(StrBuf *Target, WCTemplputParams *TP)
 
 void tmplput_ITERATE_KEY(StrBuf *Target, WCTemplputParams *TP)
 {
-	IterateStruct *Ctx = CCTX;
+	IterateStruct *Ctx = CTX(CTX_ITERATE);
 
 	StrBufAppendBufPlain(Target, Ctx->Key, Ctx->KeyLen, 0);
 }
@@ -2017,19 +2214,19 @@ void tmplput_ITERATE_KEY(StrBuf *Target, WCTemplputParams *TP)
 
 void tmplput_ITERATE_LASTN(StrBuf *Target, WCTemplputParams *TP)
 {
-	IterateStruct *Ctx = CCTX;
+	IterateStruct *Ctx = CTX(CTX_ITERATE);
 	StrBufAppendPrintf(Target, "%d", Ctx->n);
 }
 
 int conditional_ITERATE_FIRSTN(StrBuf *Target, WCTemplputParams *TP)
 {
-	IterateStruct *Ctx = CCTX;
+	IterateStruct *Ctx = CTX(CTX_ITERATE);
 	return Ctx->n == 0;
 }
 
 int conditional_ITERATE_LASTN(StrBuf *Target, WCTemplputParams *TP)
 {
-	IterateStruct *Ctx = CCTX;
+	IterateStruct *Ctx = CTX(CTX_ITERATE);
 	return Ctx->LastN;
 }
 
@@ -2038,59 +2235,52 @@ int conditional_ITERATE_LASTN(StrBuf *Target, WCTemplputParams *TP)
 /*-----------------------------------------------------------------------------
  *                      Conditionals
  */
-int EvaluateConditional(StrBuf *Target, int Neg, int state, WCTemplputParams *TP)
+int EvaluateConditional(StrBuf *Target, int Neg, int state, WCTemplputParams **TPP)
 {
 	ConditionalStruct *Cond;
 	int rc = 0;
 	int res;
+	WCTemplputParams *TP = *TPP;
 
 	if ((TP->Tokens->Params[0]->len == 1) &&
 	    (TP->Tokens->Params[0]->Start[0] == 'X'))
-		return (state != 0)?TP->Tokens->Params[1]->lvalue:0;
+	{
+		return - (TP->Tokens->Params[1]->lvalue);
+	}
 	    
 	Cond = (ConditionalStruct *) TP->Tokens->PreEval;
 	if (Cond == NULL) {
 		LogTemplateError(
 			Target, "Conditional", ERR_PARM1, TP,
 			"unknown!");
-		return 1;
+		return 0;
 	}
 
 	if (!CheckContext(Target, &Cond->Filter, TP, "Conditional")) {
 		return 0;
 	}
+
 	res = Cond->CondF(Target, TP);
 	if (res == Neg)
 		rc = TP->Tokens->Params[1]->lvalue;
+
 	if (LoadTemplates > 5) 
 		syslog(LOG_DEBUG, "<%s> : %d %d==%d\n", 
 			ChrPtr(TP->Tokens->FlatToken), 
 			rc, res, Neg);
+
+	if (TP->Sub != NULL)
+	{
+		*TPP = TP->Sub;
+	}
 	return rc;
 }
 
-void RegisterConditional(const char *Name, long len, 
-			 int nParams,
-			 WCConditionalFunc CondF, 
-			 int ContextRequired)
-{
-	ConditionalStruct *Cond;
-
-	Cond = (ConditionalStruct*)malloc(sizeof(ConditionalStruct));
-	memset(Cond, 0, sizeof(ConditionalStruct));
-	Cond->PlainName = Name;
-	Cond->Filter.nMaxArgs = nParams;
-	Cond->Filter.nMinArgs = nParams;
-	Cond->CondF = CondF;
-	Cond->Filter.ContextType = ContextRequired;
-	Cond->Filter.ControlContextType = CTX_NONE;
-	Put(Conditionals, Name, len, Cond, NULL);
-}
-
-void RegisterControlConditional(const char *Name, long len, 
+void RegisterContextConditional(const char *Name, long len, 
 				int nParams,
 				WCConditionalFunc CondF, 
-				int ControlContextRequired)
+				WCConditionalFunc ExitCtxCond,
+				int ContextRequired)
 {
 	ConditionalStruct *Cond;
 
@@ -2100,8 +2290,8 @@ void RegisterControlConditional(const char *Name, long len,
 	Cond->Filter.nMaxArgs = nParams;
 	Cond->Filter.nMinArgs = nParams;
 	Cond->CondF = CondF;
-	Cond->Filter.ContextType = CTX_NONE;
-	Cond->Filter.ControlContextType = ControlContextRequired;
+	Cond->CondExitCtx = ExitCtxCond;
+	Cond->Filter.ContextType = ContextRequired;
 	Put(Conditionals, Name, len, Cond, NULL);
 }
 
@@ -2155,11 +2345,11 @@ HashList *Defines;
  */
 void tmplput_ContextString(StrBuf *Target, WCTemplputParams *TP)
 {
-	StrBufAppendTemplate(Target, TP, (StrBuf*)CTX, 0);
+	StrBufAppendTemplate(Target, TP, (StrBuf*)CTX(CTX_STRBUF), 0);
 }
 int ConditionalContextStr(StrBuf *Target, WCTemplputParams *TP)
 {
-	StrBuf *TokenText = (StrBuf*) CTX;
+	StrBuf *TokenText = (StrBuf*) CTX((CTX_STRBUF));
 	const char *CompareToken;
 	long len;
 
@@ -2169,7 +2359,7 @@ int ConditionalContextStr(StrBuf *Target, WCTemplputParams *TP)
 
 void tmplput_ContextStringArray(StrBuf *Target, WCTemplputParams *TP)
 {
-	HashList *Arr = (HashList*) CTX;
+	HashList *Arr = (HashList*) CTX(CTX_STRBUFARR);
 	void *pV;
 	int val;
 
@@ -2181,7 +2371,7 @@ void tmplput_ContextStringArray(StrBuf *Target, WCTemplputParams *TP)
 }
 int ConditionalContextStrinArray(StrBuf *Target, WCTemplputParams *TP)
 {
-	HashList *Arr = (HashList*) CTX;
+	HashList *Arr = (HashList*) CTX(CTX_STRBUFARR);
 	void *pV;
 	int val;
 	const char *CompareToken;
@@ -2223,10 +2413,12 @@ void tmpl_do_boxed(StrBuf *Target, WCTemplputParams *TP)
 		}
 	}
 	/* else TODO error? logging? */
-	memcpy (&SubTP, TP, sizeof(WCTemplputParams));
-	SubTP.Context = Headline;
-	SubTP.Filter.ContextType = CTX_STRBUF;
-	DoTemplate(HKEY("box_begin"), Target, &SubTP);
+
+	StackContext (TP, &SubTP, Headline, CTX_STRBUF, 0, NULL);
+	{
+		DoTemplate(HKEY("box_begin"), Target, &SubTP);
+	}
+	UnStackContext(&SubTP);
 	DoTemplate(TKEY(0), Target, TP);
 	DoTemplate(HKEY("box_end"), Target, TP);
 	FreeStrBuf(&Headline);
@@ -2249,7 +2441,6 @@ int preeval_do_tabbed(WCTemplateToken *Token)
 	long len;
 	int i, nTabs;
 
-
 	memset(&TPP, 0, sizeof(WCTemplputParams));
 	TP = &TPP;
 	TP->Tokens = Token;
@@ -2312,7 +2503,6 @@ void tmpl_do_tabbed(StrBuf *Target, WCTemplputParams *TP)
 	WCTemplputParams SubTP;
 
 	memset(&TS, 0, sizeof(tab_struct));
-	memcpy (&SubTP, &TP, sizeof(WCTemplputParams));
 
 	nTabs = ntabs = TP->Tokens->nParameters / 2;
 	TabNames = (StrBuf **) malloc(ntabs * sizeof(StrBuf*));
@@ -2339,33 +2529,34 @@ void tmpl_do_tabbed(StrBuf *Target, WCTemplputParams *TP)
 			nTabs --;
 		}
 	}
-	memcpy (&SubTP, TP, sizeof(WCTemplputParams));
-	SubTP.Filter.ControlContextType = CTX_TAB;
-	SubTP.ControlContext = &TS;
-
-	StrTabbedDialog(Target, nTabs, TabNames);
-	for (i = 0; i < ntabs; i++) {
-		memset(&TS, 0, sizeof(tab_struct));
-		TS.CurrentTab = i;
-		TS.TabTitle = TabNames[i];
-		StrBeginTab(Target, i, nTabs, TabNames);
-		DoTemplate(TKEY(i * 2 + 1), Target, &SubTP);
-		StrEndTab(Target, i, nTabs);
+	StackContext (TP, &SubTP, &TS, CTX_TAB, 0, NULL);
+	{
+		StrTabbedDialog(Target, nTabs, TabNames);
+		for (i = 0; i < ntabs; i++) {
+			memset(&TS, 0, sizeof(tab_struct));
+			TS.CurrentTab = i;
+			TS.TabTitle = TabNames[i];
+			StrBeginTab(Target, i, nTabs, TabNames);
+			DoTemplate(TKEY(i * 2 + 1), Target, &SubTP);
+			StrEndTab(Target, i, nTabs);
+		}
+		for (i = 0; i < ntabs; i++) 
+			FreeStrBuf(&TabNames[i]);
+		free(TabNames);
 	}
-	for (i = 0; i < ntabs; i++) 
-		FreeStrBuf(&TabNames[i]);
+	UnStackContext(&SubTP);
 }
 
 void tmplput_TAB_N(StrBuf *Target, WCTemplputParams *TP)
 {
-	tab_struct *Ctx = CCTX;
+	tab_struct *Ctx = CTX(CTX_TAB);
 
 	StrBufAppendPrintf(Target, "%d", Ctx->CurrentTab);
 }
 
 void tmplput_TAB_TITLE(StrBuf *Target, WCTemplputParams *TP)
 {
-	tab_struct *Ctx = CCTX;
+	tab_struct *Ctx = CTX(CTX_TAB);
 	StrBufAppendTemplate(Target, TP, Ctx->TabTitle, 0);
 }
 
@@ -2379,7 +2570,7 @@ void RegisterSortFunc(const char *name, long len,
 		      CompareFunc Forward, 
 		      CompareFunc Reverse, 
 		      CompareFunc GroupChange, 
-		      long ContextType)
+		      CtxType ContextType)
 {
 	SortStruct *NewSort;
 
@@ -2652,7 +2843,7 @@ void tmplput_SORT_ORDER(StrBuf *Target, WCTemplputParams *TP)
 
 void tmplput_long_vector(StrBuf *Target, WCTemplputParams *TP)
 {
-	long *LongVector = (long*) CTX;
+	long *LongVector = (long*) CTX(CTX_LONGVECTOR);
 
 	if ((TP->Tokens->Params[0]->Type == TYPE_LONG) && 
 	    (TP->Tokens->Params[0]->lvalue <= LongVector[0]))
@@ -2695,7 +2886,7 @@ void dbg_print_longvector(long *LongVector)
 
 int ConditionalLongVector(StrBuf *Target, WCTemplputParams *TP)
 {
-	long *LongVector = (long*) CTX;
+	long *LongVector = (long*) CTX(CTX_LONGVECTOR);
 
 	if ((TP->Tokens->Params[2]->Type == TYPE_LONG) && 
 	    (TP->Tokens->Params[2]->lvalue <= LongVector[0])&&
@@ -2735,6 +2926,9 @@ void
 InitModule_SUBST
 (void)
 {
+	RegisterCTX(CTX_TAB);
+	RegisterCTX(CTX_ITERATE);
+
 	memset(&NoCtx, 0, sizeof(WCTemplputParams));
 	RegisterNamespace("--", 0, 2, tmplput_Comment, NULL, CTX_NONE);
 	RegisterNamespace("SORT:ICON", 1, 2, tmplput_SORT_ICON, NULL, CTX_NONE);
@@ -2745,31 +2939,31 @@ InitModule_SUBST
 	RegisterNamespace("ITERATE", 2, 100, tmpl_iterate_subtmpl, preeval_iterate, CTX_NONE);
 	RegisterNamespace("DOBOXED", 1, 2, tmpl_do_boxed, NULL, CTX_NONE);
 	RegisterNamespace("DOTABBED", 2, 100, tmpl_do_tabbed, preeval_do_tabbed, CTX_NONE);
-	RegisterControlNS(HKEY("TAB:N"), 0, 0, tmplput_TAB_N, CTX_TAB);
-	RegisterControlNS(HKEY("TAB:SUBJECT"), 0, 1, tmplput_TAB_TITLE, CTX_TAB);
+	RegisterNamespace("TAB:N", 0, 0, tmplput_TAB_N, NULL, CTX_TAB);
+	RegisterNamespace("TAB:SUBJECT", 0, 1, tmplput_TAB_TITLE, NULL, CTX_TAB);
 
 
 	RegisterNamespace("LONGVECTOR", 1, 1, tmplput_long_vector, NULL, CTX_LONGVECTOR);
 
 
-	RegisterConditional(HKEY("COND:CONTEXTSTR"), 3, ConditionalContextStr, CTX_STRBUF);
-	RegisterConditional(HKEY("COND:CONTEXTSTRARR"), 4, ConditionalContextStrinArray, CTX_STRBUFARR);
-	RegisterConditional(HKEY("COND:LONGVECTOR"), 4, ConditionalLongVector, CTX_LONGVECTOR);
+	RegisterConditional("COND:CONTEXTSTR", 3, ConditionalContextStr, CTX_STRBUF);
+	RegisterConditional("COND:CONTEXTSTRARR", 4, ConditionalContextStrinArray, CTX_STRBUFARR);
+	RegisterConditional("COND:LONGVECTOR", 4, ConditionalLongVector, CTX_LONGVECTOR);
 
 
-	RegisterControlConditional(HKEY("COND:ITERATE:ISGROUPCHANGE"), 2, 
-				   conditional_ITERATE_ISGROUPCHANGE, 
-				   CTX_ITERATE);
-	RegisterControlConditional(HKEY("COND:ITERATE:LASTN"), 2, 
-				   conditional_ITERATE_LASTN, 
-				   CTX_ITERATE);
-	RegisterControlConditional(HKEY("COND:ITERATE:FIRSTN"), 2, 
-				   conditional_ITERATE_FIRSTN, 
-				   CTX_ITERATE);
+	RegisterConditional("COND:ITERATE:ISGROUPCHANGE", 2, 
+			    conditional_ITERATE_ISGROUPCHANGE, 
+			    CTX_ITERATE);
+	RegisterConditional("COND:ITERATE:LASTN", 2, 
+			    conditional_ITERATE_LASTN, 
+			    CTX_ITERATE);
+	RegisterConditional("COND:ITERATE:FIRSTN", 2, 
+			    conditional_ITERATE_FIRSTN, 
+			    CTX_ITERATE);
 
-	RegisterControlNS(HKEY("ITERATE:ODDEVEN"), 0, 0, tmplput_ITERATE_ODDEVEN, CTX_ITERATE);
-	RegisterControlNS(HKEY("ITERATE:KEY"), 0, 0, tmplput_ITERATE_KEY, CTX_ITERATE);
-	RegisterControlNS(HKEY("ITERATE:N"), 0, 0, tmplput_ITERATE_LASTN, CTX_ITERATE);
+	RegisterNamespace("ITERATE:ODDEVEN", 0, 0, tmplput_ITERATE_ODDEVEN, NULL, CTX_ITERATE);
+	RegisterNamespace("ITERATE:KEY", 0, 0, tmplput_ITERATE_KEY, NULL, CTX_ITERATE);
+	RegisterNamespace("ITERATE:N", 0, 0, tmplput_ITERATE_LASTN, NULL, CTX_ITERATE);
 	RegisterNamespace("CURRENTFILE", 0, 1, tmplput_CURRENT_FILE, NULL, CTX_NONE);
 	RegisterNamespace("DEF:STR", 1, 1, tmplput_DefStr, NULL, CTX_NONE);
 	RegisterNamespace("DEF:VAL", 1, 1, tmplput_DefVal, NULL, CTX_NONE);
@@ -2791,6 +2985,13 @@ ServerStartModule_SUBST
 	Conditionals = NewHash(1, NULL);
 	SortHash = NewHash(1, NULL);
 	Defines = NewHash(1, NULL);
+	CtxList = NewHash(1, NULL);
+	
+	PutContextType(HKEY("CTX_NONE"), 0);
+
+	RegisterCTX(CTX_STRBUF);
+	RegisterCTX(CTX_STRBUFARR);
+	RegisterCTX(CTX_LONGVECTOR);
 }
 
 void
@@ -2812,6 +3013,7 @@ ServerShutdownModule_SUBST
 	DeleteHash(&Conditionals);
 	DeleteHash(&SortHash);
 	DeleteHash(&Defines);
+	DeleteHash(&CtxList);
 }
 
 
