@@ -53,7 +53,6 @@ int uds_connectsock(char *sockpath)
 		close(s);
 		return(-1);
 	}
-
 	return s;
 }
 
@@ -123,7 +122,28 @@ int tcp_connectsock(char *host, char *service)
 		}
 		rc = connect(s, ai->ai_addr, ai->ai_addrlen);
 		if (rc >= 0) {
+			int fdflags;
 			freeaddrinfo(res);
+
+			fdflags = fcntl(rc, F_GETFL);
+			if (fdflags < 0) {
+				syslog(LOG_ERR,
+				       "unable to get socket %d flags! %s \n",
+				       rc,
+				       strerror(errno));
+				close(rc);
+				return -1;
+			}
+			fdflags = fdflags | O_NONBLOCK;
+			if (fcntl(rc, F_SETFL, fdflags) < 0) {
+				syslog(LOG_ERR,
+				       "unable to set socket %d nonblocking flags! %s \n",
+				       rc,
+				       strerror(errno));
+				close(s);
+				return -1;
+			}
+
 			return(s);
 		}
 		else {
@@ -153,7 +173,7 @@ int serv_getln(char *strbuf, int bufsize)
 	FlushStrBuf(WCC->MigrateReadLineBuf);
 	strbuf[len] = '\0';
 #ifdef SERV_TRACE
-	syslog(LOG_DEBUG, "%3d<<<%s\n", WC->serv_sock, strbuf);
+	syslog(LOG_DEBUG, "%3d<<<%s\n", WCC->serv_sock, strbuf);
 #endif
 	return len;
 }
