@@ -30,10 +30,9 @@ var AnchorDialog = {
 			ed.selection.collapse(1);
 
 		elm = ed.dom.getParent(ed.selection.getNode(), 'A');
-		if (elm) {
-			elm.setAttribute('name', name);
+		if (elm)
 			elm.name = name;
-		} else
+		else
 			ed.execCommand('mceInsertContent', 0, ed.dom.createHTML('a', {name : name, 'class' : 'mceItemAnchor'}, ''));
 
 		tinyMCEPopup.close();
