@@ -3,7 +3,7 @@
  * persistent session to the Citadel server, handling HTTP WebCit requests as
  * they arrive and presenting a user interface.
  *
- * Copyright (c) 1996-2012 by the citadel.org team
+ * Copyright (c) 1996-2013 by the citadel.org team
  *
  * This program is open source software.  You can redistribute it and/or
  * modify it under the terms of the GNU General Public License, version 3.
@@ -314,6 +314,11 @@ void authorization_required(void)
 		PACKAGE_STRING, ChrPtr(WC->serv_info->serv_software)
 	);
 	hprintf("WWW-Authenticate: Basic realm=\"%s\"\r\n", ChrPtr(WC->serv_info->serv_humannode));
+
+	/* if this is a false cookie authentication, remove it to avoid endless loops. */
+	if (StrLength(WCC->Hdr->HR.RawCookie) > 0)
+		stuff_to_cookie(1);
+
 	hprintf("Content-Type: text/html\r\n");
 	begin_burst();
 	wc_printf("<h1>");
@@ -906,9 +911,9 @@ InitModule_WEBCIT
 	WebcitAddUrlHandler(HKEY("pop"), "", 0, pop_destination, 0);
 
 	WebcitAddUrlHandler(HKEY("401"), "", 0, authorization_required, ANONYMOUS|COOKIEUNNEEDED);
-	RegisterConditional(HKEY("COND:IMPMSG"), 0, ConditionalImportantMesage, CTX_NONE);
-	RegisterConditional(HKEY("COND:REST:DEPTH"), 0, Conditional_REST_DEPTH, CTX_NONE);
-	RegisterConditional(HKEY("COND:IS_HTTPS"), 0, Conditional_IS_HTTPS, CTX_NONE);
+	RegisterConditional("COND:IMPMSG", 0, ConditionalImportantMesage, CTX_NONE);
+	RegisterConditional("COND:REST:DEPTH", 0, Conditional_REST_DEPTH, CTX_NONE);
+	RegisterConditional("COND:IS_HTTPS", 0, Conditional_IS_HTTPS, CTX_NONE);
 
 	RegisterNamespace("CSSLOCAL", 0, 0, tmplput_csslocal, NULL, CTX_NONE);
 	RegisterNamespace("IMPORTANTMESSAGE", 0, 0, tmplput_importantmessage, NULL, CTX_NONE);
